package no.ntnu.item.arctis.examples.android.geirsag.audio.playaudio2;

import java.io.DataInputStream;
import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;

import android.media.AudioFormat;
import android.media.AudioTrack;

public class PlayAudio2 {

	private Player player = null;
	private Thread playerThread = null;

	// Instance parameter. Edit only in overview page.
	public final int streamType;
	// Instance parameter. Edit only in overview page.
	public final int sampleRateInHz;
	// Instance parameter. Edit only in overview page.
	public final int channelConfig;
	// Instance parameter. Edit only in overview page.
	public final int bufferMultiplier;
	// Instance parameter. Edit only in overview page.
	public final int audioFormat;

	// Do not edit this constructor.
	public PlayAudio2(int streamType, int sampleRateInHz, int channelConfig, int bufferMultiplier, int audioFormat) {
		this.streamType = streamType;
		this.sampleRateInHz = sampleRateInHz;
		this.channelConfig = channelConfig;
		this.bufferMultiplier = bufferMultiplier;
		this.audioFormat = audioFormat;
	}

	public void initialize(InputStream is) {
		player = new Player(is, streamType, sampleRateInHz, channelConfig, bufferMultiplier, audioFormat);
		playerThread = new Thread(player);
		playerThread.start();
	}
	

	public void pause() {
		player.setPaused(true);
	}

	public void start() {
		player.setPlaying(true);
	}

	public void stop() {
		player.setPlaying(false);
		try {
			playerThread.join();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	public void unPause() {
		player.setPaused(false);
	}

	private class Player implements Runnable {
		private final Object mutex = new Object();
		private final InputStream is;
		private int streamType;
		private int sampleRateInHz;
		private int channelConfig;
		private int bufferMultiplier;
		private int audioFormat;

		private volatile boolean isPlaying = true;
		private volatile boolean isPaused = false;

		public Player(InputStream is, int streamType, int sampleRateInHz, int channelConfig, int bufferMultiplier,
				int audioFormat) {
			super();
			this.is = is;
			this.streamType = streamType;
			this.sampleRateInHz = sampleRateInHz;
			this.channelConfig = channelConfig;
			this.bufferMultiplier = bufferMultiplier;
			this.audioFormat = AudioFormat.ENCODING_PCM_16BIT;
		}

		@Override
		public void run() {

			synchronized (mutex) {
				while (!this.isPlaying) {
					try {
						mutex.wait();
					} catch (InterruptedException e) {
						throw new IllegalStateException("Wait() interrupted!", e);
					}
				}
			}

			DataInputStream dis = new DataInputStream(is);

			// Were important
			android.os.Process.setThreadPriority(android.os.Process.THREAD_PRIORITY_URGENT_AUDIO);

			int bufferSize = AudioTrack.getMinBufferSize(sampleRateInHz, channelConfig, audioFormat) * bufferMultiplier;
			if(bufferSize <= 0)
				throw new IllegalStateException("Could not initialize buffer, error msg: " + bufferSize);
			AudioTrack audioTrack = new AudioTrack(streamType, sampleRateInHz, channelConfig, audioFormat, bufferSize,
					AudioTrack.MODE_STREAM);

			int writtenBytes = 0;
			short[] tempBuffer = new short[bufferSize];
			audioTrack.play();
			while (isPlaying) {
				synchronized (mutex) {
					if (this.isPaused) {
						try {
							mutex.wait(250);
						} catch (InterruptedException e) {
							throw new IllegalStateException("Wait() interrupted!", e);
						}
						continue;
					}
				}
				int samplesRead = 0;
				try {
					for (samplesRead = 0; samplesRead < bufferSize; ++samplesRead) {
						tempBuffer[samplesRead] = dis.readShort();
					}
				} catch (EOFException e) {
					//Not enough data to play, end of stream reached
					this.isPlaying = false;
				} catch (IOException e) {
					throw new IllegalStateException("dataInputStream.readShort()");
				}
				writtenBytes = audioTrack.write(tempBuffer, 0, samplesRead);
				if (writtenBytes == AudioTrack.ERROR_INVALID_OPERATION) {
					throw new IllegalStateException("read() returned AudioTrack.ERROR_INVALID_OPERATION");
				} else if (writtenBytes == AudioTrack.ERROR_BAD_VALUE) {
					throw new IllegalStateException("read() returned AudioTrack.ERROR_BAD_VALUE");
				}
			}

			audioTrack.flush();
			audioTrack.stop();
		}

		public int getStreamType() {
			return streamType;
		}

		public void setStreamType(int streamType) {
			this.streamType = streamType;
		}

		public int getSampleRateInHz() {
			return sampleRateInHz;
		}

		public void setSampleRateInHz(int sampleRateInHz) {
			this.sampleRateInHz = sampleRateInHz;
		}

		public int getChannelConfig() {
			return channelConfig;
		}

		public void setChannelConfig(int channelConfig) {
			this.channelConfig = channelConfig;
		}

		public int getBufferMultiplier() {
			return bufferMultiplier;
		}

		public void setBufferMultiplier(int bufferMultiplier) {
			this.bufferMultiplier = bufferMultiplier;
		}

		public int getAudioFormat() {
			return audioFormat;
		}

		public void setAudioFormat(int audioFormat) {
			this.audioFormat = audioFormat;
		}

		public boolean isPlaying() {
			return isPlaying;
		}

		public void setPlaying(boolean isPlaying) {
			this.isPlaying = isPlaying;
		}

		public boolean isPaused() {
			return isPaused;
		}

		public void setPaused(boolean isPaused) {
			this.isPaused = isPaused;
		}
	}
}