package no.ntnu.item.arctis.examples.android.geirsag.audio.recordaudio;

import no.ntnu.item.arctis.runtime.android.AndroidBlock;
import android.media.AudioRecord;

public class RecordAudio extends AndroidBlock {

	private Thread readerThread;

	private volatile AudioRecord audioRecord;

	private volatile boolean stop = false;
	
	public final int audioSource;

	// Instance parameter. Edit only in overview page.
	public final int channelConfig;

	// Instance parameter. Edit only in overview page.
	public final int audioFormat;

	// Instance parameter. Edit only in overview page.
	public final int sampleRateInHz;

	// Instance parameter. Edit only in overview page.
	public final int recordBufferMultiplier;

	// Instance parameter. Edit only in overview page.
	public final int outBufferSize;

	public boolean initialize() {
		int bufferSize = AudioRecord.getMinBufferSize(sampleRateInHz, channelConfig, audioFormat);
		if (bufferSize < 0)
			return false;
		bufferSize *= recordBufferMultiplier;
		audioRecord = new AudioRecord(audioSource, sampleRateInHz, channelConfig, audioFormat, bufferSize);

		Runnable r = new Runnable() {
			public void run() {
				stop = false;
				byte[] buffer = new byte[outBufferSize];
				while (!stop) {
					audioRecord.read(buffer, 0, outBufferSize);
					sendToBlock("DATA", buffer);
				}
			}
		};

		readerThread = new Thread(r);

		if (audioRecord.getState() == AudioRecord.STATE_INITIALIZED)
			return true;
		else
			return false;
	}

	public void startRecording() {
		if (audioRecord.getRecordingState() == AudioRecord.RECORDSTATE_STOPPED) {
			audioRecord.startRecording();
			readerThread.start();
		}
	}

	public void stopRecording() {
		if (audioRecord.getRecordingState() == AudioRecord.RECORDSTATE_RECORDING) {
			audioRecord.stop();
			stop = true;
		}
	}

	// Do not edit this constructor.
	public RecordAudio(int audioSource, int channelConfig, int audioFormat, int sampleRateInHz, int recordBufferMultiplier, int outBufferSize) {
	    this.audioSource = audioSource;
	    this.channelConfig = channelConfig;
	    this.audioFormat = audioFormat;
	    this.sampleRateInHz = sampleRateInHz;
	    this.recordBufferMultiplier = recordBufferMultiplier;
	    this.outBufferSize = outBufferSize;
	}

}
