package no.ntnu.item.arctis.examples.android.geirsag.audio.recordaudio2;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import no.ntnu.item.arctis.runtime.android.AndroidBlock;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.util.Log;

public class RecordAudio2 extends AndroidBlock {

	private Recorder recorder;
	private Thread recorderThread;

	// Instance parameter. Edit only in overview page.
	public final int audioSource;
	// Instance parameter. Edit only in overview page.
	public final int channelConfig;
	// Instance parameter. Edit only in overview page.
	public final int audioFormat;
	// Instance parameter. Edit only in overview page.
	public final int sampleRateInHz;
	// Instance parameter. Edit only in overview page.
	public final int recordBufferMultiplier;

	public void initialize(OutputStream os) {
		recorder = new Recorder(os, audioSource, channelConfig, sampleRateInHz, recordBufferMultiplier);
		recorderThread = new Thread(recorder);
		recorderThread.start();
	}

	// Do not edit this constructor.
	public RecordAudio2(int audioSource, int channelConfig, int audioFormat, int sampleRateInHz,
			int recordBufferMultiplier) {
		this.audioSource = audioSource;
		this.channelConfig = channelConfig;
		this.audioFormat = audioFormat;
		this.sampleRateInHz = sampleRateInHz;
		this.recordBufferMultiplier = recordBufferMultiplier;
	}

	private class Recorder implements Runnable {
		private int frequency;
		private int channelConfiguration;
		private int bufferMultiplier;
		private volatile boolean isPaused;
		// private File fileName;
		private volatile boolean isRecording;
		private final Object mutex = new Object();
		private final OutputStream os;

		private final int audioSource;

		// Changing the sample resolution changes sample type. byte vs. short.
		private final int audioEncoding = AudioFormat.ENCODING_PCM_16BIT;

		/**
	*
	*/
		public Recorder(OutputStream os, int audioSource, int channelConfig, int sampleRateInHz, int bufferMultiplier) {
			super();
			this.os = os;
			this.bufferMultiplier = bufferMultiplier;
			this.setFrequency(sampleRateInHz);
			this.setChannelConfiguration(channelConfig);
			this.audioSource = audioSource;
			this.setPaused(false);
		}

		public void run() {
			synchronized (mutex) {
				while (!this.isRecording) {
					try {
						mutex.wait();
					} catch (InterruptedException e) {
						throw new IllegalStateException("Wait() interrupted!", e);
					}
				}
			}

			DataOutputStream dataOutputStreamInstance = new DataOutputStream(os);
			android.os.Process.setThreadPriority(android.os.Process.THREAD_PRIORITY_URGENT_AUDIO);
			int bufferRead = 0;
			int bufferSize = bufferMultiplier
					* AudioRecord.getMinBufferSize(this.getFrequency(), this.getChannelConfiguration(),
							this.getAudioEncoding());
			AudioRecord recordInstance = new AudioRecord(this.audioSource, this.getFrequency(),
					this.getChannelConfiguration(), this.getAudioEncoding(), bufferSize);
			short[] tempBuffer = new short[bufferSize];
			recordInstance.startRecording();

			while (this.isRecording) {
				synchronized (mutex) {
					if (this.isPaused) {
						try {
							mutex.wait(250);
						} catch (InterruptedException e) {
							throw new IllegalStateException("Wait() interrupted!", e);
						}
						continue;
					}
				}

				bufferRead = recordInstance.read(tempBuffer, 0, bufferSize);
				if (bufferRead == AudioRecord.ERROR_INVALID_OPERATION) {
					throw new IllegalStateException("read() returned AudioRecord.ERROR_INVALID_OPERATION");
				} else if (bufferRead == AudioRecord.ERROR_BAD_VALUE) {
					throw new IllegalStateException("read() returned AudioRecord.ERROR_BAD_VALUE");
				}
				try {
					for (int idxBuffer = 0; idxBuffer < bufferRead; ++idxBuffer) {
						dataOutputStreamInstance.writeShort(tempBuffer[idxBuffer]);
					}
				} catch (IOException e) {
					Log.e("RECORDER", "IllegalStateException", new IllegalStateException(
							"dataOutputStreamInstance.writeShort(curVal)"));
				}

			}

			recordInstance.stop();
			try {
				os.flush();
				os.close();
			} catch (IOException e) {
				Log.e("RECORDER", "IllegalStateException", new IllegalStateException("Cannot close output stream."));
			}
		}

		public void setRecording(boolean isRecording) {
			synchronized (mutex) {
				this.isRecording = isRecording;
				if (this.isRecording) {
					mutex.notify();
				}
			}
		}

		public boolean isRecording() {
			synchronized (mutex) {
				return isRecording;
			}
		}

		public void setFrequency(int frequency) {
			this.frequency = frequency;
		}

		public int getFrequency() {
			return frequency;
		}

		public void setChannelConfiguration(int channelConfiguration) {
			this.channelConfiguration = channelConfiguration;
		}

		public int getChannelConfiguration() {
			return channelConfiguration;
		}

		public int getAudioEncoding() {
			return audioEncoding;
		}

		public void setPaused(boolean isPaused) {
			synchronized (mutex) {
				this.isPaused = isPaused;
			}
		}

		public boolean isPaused() {
			synchronized (mutex) {
				return isPaused;
			}
		}

		public void togglePause() {
			synchronized (mutex) {
				this.isPaused = !this.isPaused;
			}
		}

	}

	public void pause() {
		recorder.togglePause();
	}

	public void start() {
		recorder.setRecording(true);
	}

	public void stop() {
		recorder.setRecording(false);
		try {
			recorderThread.join();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

}
