package no.ntnu.item.arctis.examples.android.geirsag.file.writefile;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import no.ntnu.item.arctis.runtime.android.AndroidBlock;

public class WriteFile extends AndroidBlock {

	private SaveThread saveThread;

	public void startSave(SaveFileBundle bundle) {
		if (saveThread != null && saveThread.isAlive()) {
			saveThread.stopSaving();
			try {
				saveThread.join();
			} catch (InterruptedException e) {
			}
		}
		saveThread = new SaveThread(bundle);
		saveThread.start();
	}

	public void stopSaving() {
		if (saveThread != null && saveThread.isAlive()) {
			saveThread.stopSaving();
		}
	}

	private class SaveThread extends Thread {

		static final int BUFFER_SIZE = 8192;
		File file;
		InputStream inputStream;
		volatile boolean isSaving;

		SaveThread(SaveFileBundle bundle) {
			file = bundle.file;
			inputStream = bundle.inputStream;
		}

		@Override
		public void run() {
			isSaving = true;
			android.os.Process.setThreadPriority(android.os.Process.THREAD_PRIORITY_URGENT_AUDIO);
			BufferedOutputStream bos = null;
			byte[] buffer = new byte[BUFFER_SIZE];
			try {
				if (!file.exists() || file.delete()) {
					file.getParentFile().mkdirs();
					file.createNewFile();
				} else
					throw new IOException("Could not create new file: " + file);
				bos = new BufferedOutputStream(new FileOutputStream(file));
				sendToBlock("WRITE_TO_FILE_STARTED");
			} catch (IOException e) {
				e.printStackTrace();
				isSaving = false;
				sendToBlock("WRITE_TO_FILE_FAILED", e);
				return;
			}
			int count;
			while (isSaving) {
				try {
					count = inputStream.read(buffer);
					if (count == -1)
						isSaving = false;
					else
						bos.write(buffer, 0, count);
				} catch (IOException e) {
					e.printStackTrace();
					isSaving = false;
				}
			}
			closeStreams(bos);
			sendToBlock("WRITE_TO_FILE_FINISHED");
		}

		private void closeStreams(BufferedOutputStream bos) {
			try {
				inputStream.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			if (bos == null)
				return;
			try {
				bos.flush();
			} catch (IOException e) {
				e.printStackTrace();
			}
			try {
				bos.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		public void stopSaving() {
			isSaving = false;
		}
	}
}