package no.ntnu.item.arctis.examples.android.geirsag.invome.contactsui;

import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;

import no.ntnu.item.arctis.android.R;
import no.ntnu.item.arctis.examples.android.geirsag.invome.contactlist.ContactListActivity;
import no.ntnu.item.arctis.examples.android.geirsag.invome.contactlist.ContactListAdapter;
import no.ntnu.item.arctis.examples.android.geirsag.invome.objects.ChatPresence;
import no.ntnu.item.arctis.examples.android.geirsag.invome.objects.VoiceContact;
import no.ntnu.item.arctis.examples.android.geirsag.xmpp.objects.CustomMessage;
import no.ntnu.item.arctis.runtime.android.AndroidBlock;
import android.app.Activity;
import android.content.ContentResolver;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListAdapter;
import android.widget.ListView;

public class ContactsUI extends AndroidBlock {

	private ArrayAdapter<VoiceContact> listAdapter;
	private ListView listView;
	private Map<String, ChatPresence> pendingUpdates = new HashMap<String, ChatPresence>();

	private static Comparator<VoiceContact> comparator = new Comparator<VoiceContact>() {

		@Override
		public int compare(VoiceContact object1, VoiceContact object2) {
			int presenceDiff = object1.getPresence().compareTo(object2.getPresence());
			if (presenceDiff != 0)
				return presenceDiff;
			else {
				String dispName1 = object1.getChatUser().nickName != null ? object1.getChatUser().nickName : object1
						.getChatUser().userID;
				String dispName2 = object2.getChatUser().nickName != null ? object2.getChatUser().nickName : object2
						.getChatUser().userID;
				return dispName1.compareToIgnoreCase(dispName2);
			}
		}
	};

	private Activity activity;

	public Class getActivityClass() {
		return ContactListActivity.class;
	}

	public void registerListeners(Activity a) {
		activity = a;
		((ContactListActivity) a).setParentID(blockID);
		AndroidBlock.setCurrentActivityContext(a, blockID);

		Runnable r = new Runnable() {
			public void run() {
				listAdapter = new ContactListAdapter(activity.getApplicationContext(), R.layout.single_contact);
				listView = (ListView) activity.findViewById(android.R.id.list);
				listView.setAdapter(listAdapter);
				listView.setOnItemClickListener(new AdapterView.OnItemClickListener() {

					public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
						VoiceContact contact = listAdapter.getItem(position);
						if (contact.isVoiceSupported())
							sendToBlock("START_MESSAGE", contact);
					}
				});

			}
		};
		activity.runOnUiThread(r);

		/*
		 * Button disconnectButton =
		 * (Button)activity.findViewById(R.id.button_disconnect);
		 * disconnectButton.setOnClickListener(new View.OnClickListener() {
		 * 
		 * public void onClick(View v) { sendToBlock("DISCONNECT"); } });
		 */
	}

	public String getDialogText(VoiceContact user) {
		String nick = user.getChatUser().nickName != null ? user.getChatUser().nickName : user.getChatUser().userID;
		return "Recording message to " + nick + "...";
	}

	public void updatePresence(ChatPresence presence) {

		VoiceContact contact = findContact(listAdapter, presence.getUserId());
		if (contact != null) {
			log("Contacts UI: Setting presence: " + presence);
			contact.setPresence(presence.getStatus());
			refreshList();
		} else {
			log("Contacts UI: Contact not found: " + presence.getUserId());
			pendingUpdates.put(presence.getUserId(), presence);
		}
	}

	public void updateRoster(final Collection<VoiceContact> updatedContacts) {
		updatePhotos(updatedContacts);
		Runnable r = new Runnable() {
			public void run() {
				log("Contacts UI: Adding contacts to list");
				listAdapter.clear();
				for (VoiceContact c : updatedContacts) {
					String user = c.getChatUser().userID;
					if(pendingUpdates.containsKey(user)){
						c.setPresence(pendingUpdates.get(user).getStatus());
						pendingUpdates.remove(user);
					}
					listAdapter.add(c);
				}
			}
		};
		activity.runOnUiThread(r);
		refreshList();
	}

	private void updatePhotos(Collection<VoiceContact> updatedContacts) {
		ContentResolver cr = getContext().getContentResolver();
		// String[] projection = { ContactsContract.Contacts.
		// Cursor cursor = cr.query(ContactsContract.Contacts.CONTENT_URI, ,
		// selection, selectionArgs, sortOrder)
	}

	public VoiceContact findContact(ListAdapter adapter, String user) {
		if (adapter != null) {
			for (int i = 0; i < adapter.getCount(); i++) {
				VoiceContact tempContact = listAdapter.getItem(i);
				if (tempContact.getChatUser().userID.contentEquals(user)) {
					return tempContact;
				}
			}
		}
		return null;
	}

	public void updateIp(CustomMessage ipResponse) {
		if (!(ipResponse.getName().equalsIgnoreCase("IP_RESPONSE") || ipResponse.getName().equalsIgnoreCase(
				"IP_REQUEST"))) {
			log("Contacts UI: Received unexpected message of type: " + ipResponse.getName());
			return;
		}

		VoiceContact contact = findContact(listAdapter, ipResponse.getFromUser());
		if (contact != null) {
			contact.setIp((InetAddress) ipResponse.getValue());
			refreshList();
		} else
			log("Contacts UI: Contact not found: " + ipResponse.getFromUser());

	}

	public void updatePresences(Collection<ChatPresence> presences) {
		for (ChatPresence presence : presences) {

			VoiceContact contact = findContact(listAdapter, presence.getUserId());
			if (contact != null) {
				contact.setPresence(presence.getStatus());
			} else
				log("Contacts UI: Contact not found: " + presence.getUserId());
		}

		refreshList();
	}

	private void refreshList() {
		Runnable r = new Runnable() {
			@Override
			public void run() {
				log("Contacts UI: Refreshing contact list");
				listAdapter.sort(comparator);
				listAdapter.notifyDataSetChanged();
			}
		};
		activity.runOnUiThread(r);
	}

	public String msgReceived(InetSocketAddress address) {
		return String.format("Started receiving message from %s", findContactByIp(address));
	}

	public String msgStopped(InetSocketAddress address) {
		return String.format("Message from %s stopped", findContactByIp(address));
	}

	private VoiceContact findContactByIp(InetSocketAddress address) {
		for (int i = 0; i < listAdapter.getCount(); i++) {
			VoiceContact aContact = listAdapter.getItem(i);
			if (aContact.getIp().equals(address.getAddress()))
				return aContact;
		}
		return null;
	}
}
