package no.ntnu.item.arctis.examples.android.geirsag.invome2.contactsui2;

import java.io.File;
import java.io.FilenameFilter;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.sql.Date;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;

import no.ntnu.item.arctis.android.R;
import no.ntnu.item.arctis.examples.android.geirsag.invome.contactlist.ContactListActivity;
import no.ntnu.item.arctis.examples.android.geirsag.invome.contactlist.ContactListAdapter;
import no.ntnu.item.arctis.examples.android.geirsag.invome.objects.ChatPresence;
import no.ntnu.item.arctis.examples.android.geirsag.invome.objects.ChatUser;
import no.ntnu.item.arctis.examples.android.geirsag.invome.objects.VoiceContact;
import no.ntnu.item.arctis.examples.android.geirsag.invome2.constants.Constants;
import no.ntnu.item.arctis.examples.android.geirsag.invome2.objects.MessageFileBundle;
import no.ntnu.item.arctis.examples.android.geirsag.xmpp.objects.CustomMessage;
import no.ntnu.item.arctis.runtime.android.AndroidBlock;

import org.jivesoftware.smack.util.StringUtils;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.os.Environment;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.ArrayAdapter;
import android.widget.ListAdapter;
import android.widget.ListView;
import android.widget.Toast;

public class ContactsUI2 extends AndroidBlock {

	private static final String DELETE_ALL_MESSAGES = "Delete All Messages";
	private static final String PLAY_RECORDED_MESSAGE = "Play Recorded Message";
	private static final String SEND_MESSAGE = "Send Voice Message";
	private static final String SEND_EMAIL = "Send Email";
	private static final String START_CHAT = "Start Chat";

	public class UserFilter implements FilenameFilter {

		String user;

		public UserFilter(String user) {
			this.user = user;
		}

		@Override
		public boolean accept(File dir, String filename) {
			return filename.startsWith(user);
		}

	}

	private static final String TAG = "Contacts UI";
	private ArrayAdapter<VoiceContact> listAdapter;
	private ListView listView;
	private Map<String, ChatPresence> pendingUpdates = new HashMap<String, ChatPresence>();

	private static Comparator<VoiceContact> comparator = new Comparator<VoiceContact>() {

		@Override
		public int compare(VoiceContact object1, VoiceContact object2) {
			int presenceDiff = object1.getPresence().compareTo(object2.getPresence());
			if (presenceDiff != 0)
				return presenceDiff;
			else if (object1.isVoiceSupported() && !object2.isVoiceSupported())
				return -1;
			else if (!object1.isVoiceSupported() && object2.isVoiceSupported())
				return 1;
			else if (object1.getMessages() > 0 && object2.getMessages() == 0)
				return -1;
			else if (object1.getMessages() == 0 && object2.getMessages() > 0)
				return 1;
			else {
				String dispName1 = object1.getChatUser().nickName != null ? object1.getChatUser().nickName : object1
						.getChatUser().userID;
				String dispName2 = object2.getChatUser().nickName != null ? object2.getChatUser().nickName : object2
						.getChatUser().userID;
				return dispName1.compareToIgnoreCase(dispName2);
			}
		}
	};

	private Activity activity;

	public Class getActivityClass() {
		return ContactListActivity.class;
	}

	public void registerListeners(Activity a) {
		activity = a;
		((ContactListActivity) a).setParentID(blockID);
		AndroidBlock.setCurrentActivityContext(a, blockID);

		Runnable r = new Runnable() {
			public void run() {
				listAdapter = new ContactListAdapter(activity.getApplicationContext(), R.layout.single_contact);
				listView = (ListView) activity.findViewById(android.R.id.list);
				listView.setAdapter(listAdapter);
				listView.setOnItemClickListener(new AdapterView.OnItemClickListener() {

					public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
						VoiceContact contact = listAdapter.getItem(position);
						showDialogOptions(contact);
					}
				});
				listView.setOnItemLongClickListener(new OnItemLongClickListener() {

					@Override
					public boolean onItemLongClick(AdapterView<?> parent, View view, int position, long id) {
						final VoiceContact contact = listAdapter.getItem(position);
						showDialogOptions(contact);
						return true;
					}
				});

			}
		};
		activity.runOnUiThread(r);
	}

	private void showDialogOptions(final VoiceContact contact) {
		final boolean hasMessages = contact.getMessages() > 0;
		final boolean voiceSupported = contact.isVoiceSupported();

		CharSequence[] options;
		if (!hasMessages && !voiceSupported)
			options = new CharSequence[] { START_CHAT, SEND_EMAIL };
		else if (hasMessages && voiceSupported)
			options = new CharSequence[] { SEND_MESSAGE, START_CHAT, SEND_EMAIL, PLAY_RECORDED_MESSAGE,
					DELETE_ALL_MESSAGES };
		else if (hasMessages && !voiceSupported)
			options = new CharSequence[] { START_CHAT, SEND_EMAIL, PLAY_RECORDED_MESSAGE, DELETE_ALL_MESSAGES };
		else
			options = new CharSequence[] { SEND_MESSAGE, START_CHAT, SEND_EMAIL };

		AlertDialog.Builder builder = new AlertDialog.Builder(getCurrentActivityContext());
		builder.setTitle(contact.getChatUser().nickName != null ? contact.getChatUser().nickName : contact
				.getChatUser().userID);
		builder.setItems(options, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				if (!voiceSupported)
					which++;

				switch (which) {
				case 0:
					sendToBlock("START_MESSAGE", contact);
					break;
				case 1:
					startChat(contact);
					break;
				case 2:
					sendEmail(contact);
					break;
				case 3:
					showMessages(contact);
					break;
				case 4:
					deleteMessages(contact);
					break;
				default:
					break;
				}
			}

		}).show();
	}

	private void sendEmail(VoiceContact contact) {
		final Intent emailIntent = new Intent(android.content.Intent.ACTION_SEND);
		emailIntent.setType("plain/text");
		emailIntent.putExtra(android.content.Intent.EXTRA_EMAIL, new String[] { contact.getChatUser().userID });
		getCurrentActivityContext().startActivity(Intent.createChooser(emailIntent, SEND_EMAIL));
	}

	private void startChat(VoiceContact contact) {
		Uri imUri = new Uri.Builder().scheme("imto").authority("gtalk").appendPath(contact.getChatUser().userID)
				.build();
		Intent intent = new Intent(Intent.ACTION_SENDTO, imUri);
		getCurrentActivityContext().startActivity(Intent.createChooser(intent, START_CHAT));
	}

	private void showMessages(VoiceContact contact) {
		final File[] messages = getFiles(contact);
		String[] dates = new String[messages.length];
		for (int i = 0; i < messages.length; i++) {
			dates[i] = String.format("%1$TF %1$TT", new Date(messages[i].lastModified()));
		}
		AlertDialog.Builder builder = new AlertDialog.Builder(getCurrentActivityContext());
		builder.setItems(dates, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				sendToBlock("PLAY_MESSAGE", messages[which]);
			}
		}).show();
	}

	private void deleteMessages(VoiceContact contact) {
		File[] files = getFiles(contact);
		for (File file : files) {
			file.delete();
		}
		contact.setMessages(0);
		refreshList();
	}

	private File[] getFiles(VoiceContact contact) {
		File dir = new File(Environment.getExternalStorageDirectory(), Constants.FILES_DIR);
		FilenameFilter filter = new UserFilter(contact.getChatUser().userID);
		return dir.listFiles(filter);
	}

	public void updatePresence(ChatPresence presence) {

		VoiceContact contact = findContact(listAdapter, presence.getUserId());
		String bareAddress = StringUtils.parseBareAddress(presence.getUserId());
		if (contact != null) {
			log("Contacts UI: Setting presence: " + presence);
			pendingUpdates.remove(bareAddress);
			contact.setPresence(presence.getStatus());
			contact.setStatus(presence.getMessage());

			contact.setIp(null);
			refreshList();
		} else {
			log("Contacts UI: Contact not found: " + presence.getUserId());
			pendingUpdates.put(bareAddress, presence);
		}
	}

	public void updateRoster(final Collection<VoiceContact> updatedContacts) {
		updatePhotos(updatedContacts);
		updateMessages(updatedContacts);
		Runnable r = new Runnable() {
			public void run() {
				log("Contacts UI: Adding contacts to list");
				listAdapter.clear();
				for (VoiceContact c : updatedContacts) {
					String user = StringUtils.parseBareAddress(c.getChatUser().userID);
					if (pendingUpdates.containsKey(user)) {
						c.setPresence(pendingUpdates.get(user).getStatus());
						pendingUpdates.remove(user);
					}
					listAdapter.add(c);
				}
			}
		};
		activity.runOnUiThread(r);
		refreshList();
	}

	private void updateMessages(Collection<VoiceContact> updatedContacts) {

		File directory = new File(Environment.getExternalStorageDirectory(), Constants.FILES_DIR);
		if (!directory.exists() || !directory.isDirectory())
			return;

		for (VoiceContact voiceContact : updatedContacts) {
			String user = voiceContact.getChatUser().userID;
			FilenameFilter filter = new UserFilter(user);
			File[] messages = directory.listFiles(filter);
			voiceContact.setMessages(messages.length);
		}
	}

	private void updatePhotos(Collection<VoiceContact> updatedContacts) {

	}

	public VoiceContact findContact(ListAdapter adapter, String user) {
		if (adapter != null) {
			for (int i = 0; i < adapter.getCount(); i++) {
				VoiceContact tempContact = listAdapter.getItem(i);
				if (tempContact.getChatUser().userID.contentEquals(user)) {
					return tempContact;
				}
			}
		}
		return null;
	}

	public void invomeUpdate(CustomMessage invomeMsg) {
		if (!(invomeMsg.getName().equalsIgnoreCase("IP_RESPONSE") || invomeMsg.getName().equalsIgnoreCase("IP_REQUEST") || invomeMsg
				.getName().equalsIgnoreCase("INVOME_BYE"))) {
			log("Contacts UI: Received unexpected message of type: " + invomeMsg.getName());
			return;
		}

		VoiceContact contact = findContact(listAdapter, invomeMsg.getFromUser());
		if (contact != null) {
			if (invomeMsg.getName().equals("INVOME_BYE"))
				contact.setIp(null);
			else
				contact.setIp((InetAddress) invomeMsg.getValue());
			refreshList();
		} else
			log("Contacts UI: Contact not found: " + invomeMsg.getFromUser());
	}

	public void msgSaved(MessageFileBundle bundle) {
		VoiceContact contact = findContactByIp(bundle.rtpAddress);
		if (contact == null)
			return;
		contact.setMessages(contact.getMessages() + 1);
		refreshList();
	}

	private void refreshList() {
		Runnable r = new Runnable() {
			@Override
			public void run() {
				log("Contacts UI: Refreshing contact list");
				listAdapter.sort(comparator);
				listAdapter.notifyDataSetChanged();
			}
		};
		activity.runOnUiThread(r);
	}

	private VoiceContact findContactByIp(InetSocketAddress address) {
		if (address == null) {
			Log.w(TAG, "FindContactByIp received null address");
			return null;
		}
		for (int i = 0; i < listAdapter.getCount(); i++) {
			VoiceContact aContact = listAdapter.getItem(i);
			if (aContact.getIp().equals(address.getAddress()))
				return aContact;
		}
		return null;
	}

}