package no.ntnu.item.arctis.examples.android.geirsag.invome2.togui;

import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;

import no.ntnu.item.arctis.examples.android.geirsag.invome.objects.ChatPresence;
import no.ntnu.item.arctis.examples.android.geirsag.invome.objects.VoiceContact;
import no.ntnu.item.arctis.examples.android.geirsag.invome2.constants.Constants;
import no.ntnu.item.arctis.examples.android.geirsag.invome2.objects.MessageFileBundle;
import no.ntnu.item.arctis.examples.android.geirsag.xmpp.objects.CustomMessage;
import no.ntnu.item.arctis.examples.android.geirsag.xmpp.objects.LoginBundle;
import no.ntnu.item.arctis.examples.android.geirsag.xmpp.objects.StatusBundle;
import no.ntnu.item.arctis.runtime.android.AndroidBlock;
import android.content.Intent;

public class ToGUI extends AndroidBlock implements Constants {

	private Intent receivedIntent;
	public boolean isConnected;

	public String[] getActions() {
		return new String[] { ACTION_CONNECT, ACTION_SERVICE_STATUS_REQUEST, ACTION_DISCONNECT, ACTION_SET_STATUS,
				ACTION_PLAY_MSG, ACTION_STOP_PLAYING, ACTION_GUI_ACTIVE };
	}

	public String sortIntent(Intent intent) {
		receivedIntent = intent;
		return intent.getAction();
	}

	public void statusRespond() {
		sendIntent(ACTION_SERVICE_STATUS_RESPONSE, BOOLEAN_EXTRA, isConnected);
	}

	public LoginBundle getLoginBundle() {
		return (LoginBundle) receivedIntent.getSerializableExtra(LOGIN_BUNDLE_EXTRA);
	}

	public ChatPresence getChatPresence() {
		return (ChatPresence) receivedIntent.getSerializableExtra(CHAT_PRESENCE_EXTRA);
	}

	public Collection<CustomMessage> getDisconnectMsgs() {
		return (Collection<CustomMessage>) receivedIntent.getSerializableExtra(BYE_MSG_EXTRA);
	}

	public File getFile() {
		return (File) receivedIntent.getSerializableExtra(FILE_EXTRA);
	}

	public void sendConnected() {
		isConnected = true;
		sendIntent(ACTION_CONNECTED);
	}

	public void playingStopped() {
		sendIntent(ACTION_PLAYING_STOPPED);
	}

	private void sendIntent(String action) {
		Intent intent = new Intent(action);
		getContext().sendBroadcast(intent, RECEIVE_BROADCASTS_PERMISSION);
	}

	private void sendIntent(String action, String key, Serializable extra) {
		Intent intent = new Intent(action);
		intent.putExtra(key, extra);
		getContext().sendBroadcast(intent, RECEIVE_BROADCASTS_PERMISSION);
	}

	public void msgSaved(MessageFileBundle bundle) {
		sendIntent(ACTION_MSG_SAVED, MESSAGE_FILE_BUNDLE_EXTRA, bundle);
	}

	public void playingStarted(String sender) {
		sendIntent(ACTION_PLAYING_STARTED, STRING_EXTRA, sender);
	}

	public void invomeUpdate(CustomMessage message) {
		sendIntent(ACTION_INVOME_UPDATE, CUSTOM_MESSAGE_EXTRA, message);
	}

	public void contactsUpdate(Collection<VoiceContact> contacts) {
		sendIntent(ACTION_CONTACTS_UPDATE, CONTACTS_EXTRA, (ArrayList<VoiceContact>) contacts);
	}

	public void presenceUpdate(ChatPresence presence) {
		sendIntent(ACTION_PRESENCE_UPDATE, CHAT_PRESENCE_EXTRA, presence);
	}

	public void disconnected(Exception e) {
		isConnected = false;
		StringBuilder builder = new StringBuilder("Disconnected from server");
		if (e != null)
			builder.append("\nError: Connection lost");
		sendIntent(ACTION_DISCONNECTED, STRING_EXTRA, builder.toString());
	}

	public void connectFailed(Exception e) {
		isConnected = false;
		StringBuilder builder = new StringBuilder("Could not connect to server");
		if (e != null)
			builder.append("\nError: " + e.getMessage());
		sendIntent(ACTION_CONNECT_FAILED, STRING_EXTRA, builder.toString());
	}

	public boolean getBoolean() {
		return receivedIntent.getBooleanExtra(BOOLEAN_EXTRA, false);
	}

}
