package no.ntnu.item.arctis.examples.android.geirsag.invome2.toservice;

import java.io.File;
import java.io.Serializable;
import java.net.InetSocketAddress;
import java.util.ArrayList;
import java.util.Collection;

import no.ntnu.item.arctis.examples.android.geirsag.invome.objects.ChatPresence;
import no.ntnu.item.arctis.examples.android.geirsag.invome.objects.VoiceContact;
import no.ntnu.item.arctis.examples.android.geirsag.invome2.constants.Constants;
import no.ntnu.item.arctis.examples.android.geirsag.invome2.objects.MessageFileBundle;
import no.ntnu.item.arctis.examples.android.geirsag.xmpp.objects.CustomMessage;
import no.ntnu.item.arctis.examples.android.geirsag.xmpp.objects.LoginBundle;
import no.ntnu.item.arctis.library.objects.login.Credentials;
import no.ntnu.item.arctis.runtime.android.AndroidBlock;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.IntentSender.SendIntentException;

public class ToService extends AndroidBlock implements Constants {

	private Intent receivedIntent;
	public java.lang.String disconnectedMsg;

	public void connect(Credentials creds) {
		LoginBundle loginBundle = new LoginBundle("talk.google.com", 5222, "gmail.com", creds.getUserName(),
				creds.getPassword());
		sendIntent(ACTION_CONNECT, LOGIN_BUNDLE_EXTRA, loginBundle);
	}

	public void startService() {
		Intent intent;
		try {
			intent = new Intent(getContext(),
					Class.forName("no.ntnu.item.arctis.examples.android.geirsag.invome2.invomeservice.Start"));
			getContext().startService(intent);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}

	}

	public String sortIntent(Intent intent) {
		receivedIntent = intent;
		return intent.getAction();

	}

	public String getString() {
		return receivedIntent.getStringExtra(STRING_EXTRA);
	}

	public CustomMessage getCustomMsg() {
		return (CustomMessage) receivedIntent.getSerializableExtra(CUSTOM_MESSAGE_EXTRA);
	}

	public Collection<VoiceContact> getContacts() {
		return (Collection<VoiceContact>) receivedIntent.getSerializableExtra(CONTACTS_EXTRA);
	}

	public ChatPresence getPresence() {
		return (ChatPresence) receivedIntent.getSerializableExtra(CHAT_PRESENCE_EXTRA);
	}

	public InetSocketAddress getInetAddress() {
		return (InetSocketAddress) receivedIntent.getSerializableExtra(I_NET_ADDRESS_EXTRA);
	}

	public void pollServiceStatus() {
		BroadcastReceiver receiver = new BroadcastReceiver() {

			@Override
			public void onReceive(Context context, Intent intent) {
				context.unregisterReceiver(this);
				sendToBlock("SERVICE_STATUS", intent.getBooleanExtra(BOOLEAN_EXTRA, false));
			}
		};
		getContext().registerReceiver(receiver, new IntentFilter(ACTION_SERVICE_STATUS_RESPONSE));
		getContext().sendStickyBroadcast(new Intent(ACTION_SERVICE_STATUS_REQUEST));
		Intent intent = new Intent(ACTION_GUI_ACTIVE);
		intent.putExtra(BOOLEAN_EXTRA, true);
		getContext().sendStickyBroadcast(intent);
	}

	public String[] getIntentActions() {
		return new String[] { ACTION_CONNECTED, ACTION_SERVICE_STARTED, ACTION_CONNECT_FAILED, ACTION_CONTACTS_UPDATE,
				ACTION_INVOME_UPDATE, ACTION_DISCONNECTED, ACTION_PRESENCE_UPDATE, ACTION_PLAYING_STARTED,
				ACTION_PLAYING_STOPPED, ACTION_MSG_SAVED };
	}

	private void sendIntent(String action) {
		Intent intent = new Intent(action);
		getContext().sendBroadcast(intent, RECEIVE_BROADCASTS_PERMISSION);
	}

	private void sendIntent(String action, String key, Serializable extra) {
		Intent intent = new Intent(action);
		intent.putExtra(key, extra);
		getContext().sendBroadcast(intent, RECEIVE_BROADCASTS_PERMISSION);
	}

	public void playMsg(File file) {
		sendIntent(ACTION_PLAY_MSG, FILE_EXTRA, file);
	}

	public void disconnect() {
		sendIntent(ACTION_DISCONNECT);
	}

	public void setStatus(ChatPresence presence) {
		sendIntent(ACTION_SET_STATUS, CHAT_PRESENCE_EXTRA, presence);
	}

	public MessageFileBundle getFileBundle() {
		return (MessageFileBundle) receivedIntent.getSerializableExtra(MESSAGE_FILE_BUNDLE_EXTRA);
	}

	public void stopPlaying() {
		sendIntent(ACTION_STOP_PLAYING);
	}

	public void clearDisconnectedMsg() {
		disconnectedMsg = null;
		sendIntent(ACTION_GUI_ACTIVE, BOOLEAN_EXTRA, false);
	}

}
