package no.ntnu.item.arctis.examples.android.geirsag.invome2.xmppgateway;

import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;

import no.ntnu.item.arctis.examples.android.geirsag.invome.objects.ChatPresence;
import no.ntnu.item.arctis.examples.android.geirsag.invome.objects.ChatUser;
import no.ntnu.item.arctis.examples.android.geirsag.invome.objects.VoiceContact;
import no.ntnu.item.arctis.examples.android.geirsag.invome.objects.ChatPresence.PresenceStatus;
import no.ntnu.item.arctis.examples.android.geirsag.xmpp.objects.CustomMessage;
import no.ntnu.item.arctis.examples.android.geirsag.xmpp.objects.StatusBundle;
import no.ntnu.item.arctis.runtime.Block;

import org.jivesoftware.smack.RosterEntry;
import org.jivesoftware.smack.packet.Presence;
import org.jivesoftware.smack.packet.Presence.Mode;
import org.jivesoftware.smack.util.StringUtils;

public class XMPPGateway extends Block {

	public static String IP_REQUEST = "IP_REQUEST";
	public static int IP_REQUEST_CODE = 1;
	public static int IP_RESPONSE_CODE = 2;
	public static String IP_RESPONSE = "IP_RESPONSE";
	private static CustomMessage incomingMessage;

	public Collection<VoiceContact> rosterToContacts(Collection<RosterEntry> roster) {
		Collection<VoiceContact> contacts = new ArrayList<VoiceContact>();
		for (RosterEntry e : roster) {
			contacts.add(new VoiceContact(new ChatUser(e.getName(), StringUtils.parseBareAddress(e.getUser()))));
		}
		return contacts;
	}

	public Collection<CustomMessage> createIpRequests(Collection<RosterEntry> roster) {
		Collection<CustomMessage> messages = new ArrayList<CustomMessage>();
		for (RosterEntry entry : roster) {
			messages.add(new CustomMessage(IP_REQUEST, getLocalIpAddress(), entry.getUser(), null));
		}
		return messages;
	}

	public InetAddress getLocalIpAddress() {
		try {
			for (Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces(); en.hasMoreElements();) {
				NetworkInterface intf = en.nextElement();
				for (Enumeration<InetAddress> enumIpAddr = intf.getInetAddresses(); enumIpAddr.hasMoreElements();) {
					InetAddress inetAddress = enumIpAddr.nextElement();
					if (!inetAddress.isLoopbackAddress()) {
						return inetAddress;
					}
				}
			}
		} catch (SocketException ex) {
			log("XMPP: Could not get local IP address!", ex);
		}
		return null;
	}

	public ChatPresence handlePresence(Presence presence) {
		log(String.format("VoiceSupport: Received presence update from %s: Type: %s, Mode: %s", presence.getFrom(),
				presence.getType(), presence.getMode()));
		PresenceStatus status = null;
		if (presence.getType() == Presence.Type.unavailable)
			status = PresenceStatus.unavailable;
		else if (presence.getType() == Presence.Type.available) {
			if (presence.getMode() == Presence.Mode.available || presence.getMode() == Presence.Mode.chat
					|| presence.getMode() == null)
				status = PresenceStatus.available;
			else if (presence.getMode() == Presence.Mode.dnd)
				status = PresenceStatus.busy;
			else if (presence.getMode() == Presence.Mode.away || presence.getMode() == Presence.Mode.xa)
				status = PresenceStatus.away;
		} else
			return null;
		return new ChatPresence(StringUtils.parseBareAddress(presence.getFrom()), status, presence.getStatus());
	}

	public String sortMessage(CustomMessage msg) {
		incomingMessage = msg;
		log(String.format("VoiceSupport: Received custom message '%s' from %s: %s", msg.getName(), msg.getFromUser(),
				msg.getValue().toString()));
		// Ignore requests and responses coming from own IP address
		if (msg.getValue() instanceof InetAddress && ((InetAddress) msg.getValue()).equals(getLocalIpAddress()))
			return "null";
		return msg.getName();
	}

	public CustomMessage respondToIpRequest(CustomMessage ipRequest) {
		return new CustomMessage(IP_RESPONSE, getLocalIpAddress(), ipRequest.getFromUser(), null);
	}

	public Collection<ChatPresence> handlePresences(Collection<Presence> presences) {
		Collection<ChatPresence> chatPresences = new ArrayList<ChatPresence>();
		for (Presence presence : presences) {
			ChatPresence cp = handlePresence(presence);
			if (cp != null)
				chatPresences.add(cp);
		}
		return chatPresences;
	}

	public CustomMessage presenceToIpRequest(Presence presence) {
		return new CustomMessage(IP_REQUEST, getLocalIpAddress(), StringUtils.parseBareAddress(presence.getFrom()),
				null);
	}

	public CustomMessage getMessage() {
		return incomingMessage;
	}

	public CustomMessage parse(CustomMessage msg) {
		if (msg == null)
			return null;
		return new CustomMessage(msg.getName(), msg.getValue(), StringUtils.parseBareAddress(msg.getToUser()),
				StringUtils.parseBareAddress(msg.getFromUser()));
	}

	public StatusBundle presenceToStatus(ChatPresence presence) {
		Presence.Mode mode;
		switch (presence.getStatus()) {
		case available:
			mode = Mode.available;
			break;
		case busy:
			mode = Mode.dnd;
			break;
		case away:
			mode = Mode.away;
			break;
		case unavailable:
			mode = Mode.xa;
			break;
		default:
			mode = Mode.available;
			break;
		}

		return new StatusBundle(mode, presence.getMessage());
	}
}