package no.ntnu.item.arctis.runtime.android;

import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import no.ntnu.item.arctis.android.R;
import no.ntnu.item.arctis.runtime.Logger;
import no.ntnu.item.arctis.runtime.Runtime;
import android.app.Activity;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.os.Bundle;
import android.os.Handler;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.CompoundButton;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ToggleButton;

public abstract class GeneralStartActivity3 extends Activity implements IStartActivity {

	private Runtime runtime;
	// public static HashMap<String, String> SMAdrTable = new HashMap<String,
	// String>();
	private static AndroidLogger3 logger;

	private static final int PORT_NOT_SET = -1;
	private int port = PORT_NOT_SET;

	private static final int MENU_OPEN_LOG = 1;

	private boolean RUNTIME_STATUS_ACTIVE = false;
	private boolean schedulerStarting;

	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);
		if (!isVisible()) {
			setVisible(false);
		}
		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);

		if (!isVisible()) {
			startScheduler();
			schedulerStarting = true;
		} else {
			final ToggleButton button = (ToggleButton) findViewById(R.id.togglebutton);
			button.setChecked(RUNTIME_STATUS_ACTIVE);
			button.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
				public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
					if (isChecked) {
						System.out.println("start scheduler");
						startScheduler();
						// RUNTIME_STATUS_ACTIVE = true;
						ImageView i = (ImageView) findViewById(R.id.arctislogo);
						i.setImageResource(R.drawable.arctison);
					} else {
						stopScheduler();
						// RUNTIME_STATUS_ACTIVE = false;
						ImageView i = (ImageView) findViewById(R.id.arctislogo);
						i.setImageResource(R.drawable.arctisoff);
					}
				}
			});

			ImageView i = (ImageView) findViewById(R.id.arctislogo);
			i.setImageResource(RUNTIME_STATUS_ACTIVE ? R.drawable.arctison : R.drawable.arctisoff);
		}
	}

	@Override
	protected void onResume() {
		super.onResume();
		if (!isVisible()) {
			if (schedulerStarting) {
				schedulerStarting = false;
			} else if(RUNTIME_STATUS_ACTIVE) {
				finish();
			}
		}
	}

	protected boolean isVisible() {
		return true;
	}

	public boolean onCreateOptionsMenu(Menu menu) {
		boolean result = super.onCreateOptionsMenu(menu);
		menu.add(0, MENU_OPEN_LOG, 0, "Open Log");
		return result;
	}

	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case MENU_OPEN_LOG:
			Intent intent = new Intent(this, AndroidLoggerActivity.class);
			startActivity(intent);
		}
		return super.onOptionsItemSelected(item);
	}

	private void startScheduler() {
		if (logger == null) {
			logger = new AndroidLogger3(this);
			Logger.setLogger(logger);
		}

		/*
		 * On a machine, only a single transporter can run on the same port. If
		 * another port is used than the default (proposed), the port must be
		 * specified when configuring the addresses.
		 */
		if (port == PORT_NOT_SET) {
			port = 52000;
			TextView portText = (TextView) findViewById(R.id.text_port);
			portText.setText("" + port);
		}

		InetAddress address = getLocalIpAddress();
		if (address != null) {
			TextView addText = (TextView) findViewById(R.id.ip_address);
			addText.setText("" + address.getHostAddress());
		}

		runtime = createRuntime();
		// SOAPSerializer serializer = new SOAPSerializer();
		// String endpoint = SocketTransporter.SCHEME + "://" + (address!=null ?
		// address.getHostAddress() : "localhost");
		// transporter = new SocketTransporter(port, port, endpoint, runtime,
		// serializer);
		// runtime.registerMessagedEndpoint(SocketTransporter.SCHEME,
		// transporter);
		// transporter.start();

		runtime.setContext("Android", this);
		Handler arctisUpdateHandler = new Handler();
		runtime.setContext("Handler", arctisUpdateHandler);

		runtime.setTerminationListener(new Runtime.IRuntimeTerminationListener() {
			public void runtimeStopped() {
				logger.runtimeSystemStopped();
				port = PORT_NOT_SET;
				Runnable r = new Runnable() {
					public void run() {
						TextView portText = (TextView) findViewById(R.id.text_port);
						portText.setText("-");
						ToggleButton button = (ToggleButton) findViewById(R.id.togglebutton);
						button.setChecked(false);
					}
				};
				runOnUiThread(r);
			}
		});

		runtime.start();
		RUNTIME_STATUS_ACTIVE = true;
		logger.runtimeSystemStarted();
	}

	private void stopScheduler() {
		logger.runtimeSystemStopped();
		runtime.stop();
		RUNTIME_STATUS_ACTIVE = false;
		port = PORT_NOT_SET;
		TextView portText = (TextView) findViewById(R.id.text_port);
		portText.setText("-");
	}

	protected abstract Runtime createRuntime();

	protected InetAddress getLocalIpAddress() {
		try {
			for (Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces(); en.hasMoreElements();) {
				NetworkInterface intf = en.nextElement();
				for (Enumeration<InetAddress> enumIpAddr = intf.getInetAddresses(); enumIpAddr.hasMoreElements();) {
					InetAddress inetAddress = enumIpAddr.nextElement();
					if (!inetAddress.isLoopbackAddress()) {
						return inetAddress; // .getHostAddress().toString();
					}
				}
			}
		} catch (SocketException ex) {
			ex.printStackTrace();
		}
		return null;
	}

	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		IActivityResultListener listener = map.get(requestCode);
		if (listener != null) {
			listener.onActivityResult(resultCode, data);
		}
		super.onActivityResult(requestCode, resultCode, data);
	}

	private Map<Integer, IActivityResultListener> map = new HashMap<Integer, IActivityResultListener>();

	public void registerOnActivityResultListener(int requestCode, IActivityResultListener listener) {
		map.put(requestCode, listener);
	}

	protected void onDestroy() {
		super.onDestroy();
		stopScheduler();
		Toast toast = Toast.makeText(getApplicationContext(), "Arctis is shutting down. Good bye!", Toast.LENGTH_LONG);
		toast.show();
	}

}