package no.ntnu.item.arctis.runtime.android;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;

import no.ntnu.item.arctis.runtime.Runtime;
import no.ntnu.item.arctis.android.R;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Intent;
import android.os.Handler;
import android.os.IBinder;
import android.util.Log;

public abstract class GeneralStartService extends Service implements ForegroundController {

	private static final int ID = 549613678;

	private Runtime runtime;
	// public static HashMap<String, String> SMAdrTable = new HashMap<String,
	// String>();
	// private static AndroidLogger3 logger;

	private static final int PORT_NOT_SET = -1;
	private int port = PORT_NOT_SET;

	private static final int MENU_OPEN_LOG = 1;

	private boolean RUNTIME_STATUS_ACTIVE = false;

	private static final Class<?>[] mSetForegroundSignature = new Class[] { boolean.class };
	private static final Class<?>[] mStartForegroundSignature = new Class[] { int.class, Notification.class };
	private static final Class<?>[] mStopForegroundSignature = new Class[] { boolean.class };

	private NotificationManager mNM;
	private Method mSetForeground;
	private Method mStartForeground;
	private Method mStopForeground;
	private Object[] mSetForegroundArgs = new Object[1];
	private Object[] mStartForegroundArgs = new Object[2];
	private Object[] mStopForegroundArgs = new Object[1];

	@Override
	public void moveToForeground(int iconDrawableID, String tickerText, String contentTitle, String contentText,
			Intent contentIntent) {

		// Set the icon, scrolling text and timestamp
		Notification notification = new Notification(iconDrawableID, tickerText, System.currentTimeMillis());

		// The PendingIntent to launch our activity if the user selects this
		// notification
		contentIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
		PendingIntent pendingIntent = PendingIntent.getActivity(this, 0, contentIntent, 0);

		// Set the info for the views that show in the notification panel.
		notification.setLatestEventInfo(this, contentTitle, contentText, pendingIntent);

		startForegroundCompat(ID, notification);
	}
	
	

	@Override
	public void moveToForeground(int iconDrawableID, String tickerText, String contentTitle, String contentText,
			String broadcastAction) {
		// Set the icon, scrolling text and timestamp
		Notification notification = new Notification(iconDrawableID, tickerText, System.currentTimeMillis());

		Intent intent = new Intent(broadcastAction);
		// The PendingIntent to launch our activity if the user selects this
		// notification
		PendingIntent pendingIntent = PendingIntent.getBroadcast(this, 0, intent, 0);

		// Set the info for the views that show in the notification panel.
		notification.setLatestEventInfo(this, contentTitle, contentText, pendingIntent);

		startForegroundCompat(ID, notification);
	}



	@Override
	public void moveToBackground() {
		stopForegroundCompat(ID);
	}

	void invokeMethod(Method method, Object[] args) {
		try {
			method.invoke(this, args);
		} catch (InvocationTargetException e) {
			// Should not happen.
			Log.w("ApiDemos", "Unable to invoke method", e);
		} catch (IllegalAccessException e) {
			// Should not happen.
			Log.w("ApiDemos", "Unable to invoke method", e);
		}
	}

	/**
	 * This is a wrapper around the new startForeground method, using the older
	 * APIs if it is not available.
	 */
	void startForegroundCompat(int id, Notification notification) {
		// If we have the new startForeground API, then use it.
		if (mStartForeground != null) {
			mStartForegroundArgs[0] = Integer.valueOf(id);
			mStartForegroundArgs[1] = notification;
			invokeMethod(mStartForeground, mStartForegroundArgs);
			return;
		}

		// Fall back on the old API.
		mSetForegroundArgs[0] = Boolean.TRUE;
		invokeMethod(mSetForeground, mSetForegroundArgs);
		mNM.notify(id, notification);
	}

	/**
	 * This is a wrapper around the new stopForeground method, using the older
	 * APIs if it is not available.
	 */
	void stopForegroundCompat(int id) {
		// If we have the new stopForeground API, then use it.
		if (mStopForeground != null) {
			mStopForegroundArgs[0] = Boolean.TRUE;
			try {
				mStopForeground.invoke(this, mStopForegroundArgs);
			} catch (InvocationTargetException e) {
				// Should not happen.
				Log.w("ApiDemos", "Unable to invoke stopForeground", e);
			} catch (IllegalAccessException e) {
				// Should not happen.
				Log.w("ApiDemos", "Unable to invoke stopForeground", e);
			}
			return;
		}

		// Fall back on the old API. Note to cancel BEFORE changing the
		// foreground state, since we could be killed at that point.
		mNM.cancel(id);
		mSetForegroundArgs[0] = Boolean.FALSE;
		invokeMethod(mSetForeground, mSetForegroundArgs);
	}

	public void onCreate() {
		super.onCreate();
		setupForeground();
	}

	private void setupForeground() {
		mNM = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
		try {
			mStartForeground = getClass().getMethod("startForeground", mStartForegroundSignature);
			mStopForeground = getClass().getMethod("stopForeground", mStopForegroundSignature);
		} catch (NoSuchMethodException e) {
			// Running on an older platform.
			mStartForeground = mStopForeground = null;
			return;
		}
		try {
			mSetForeground = getClass().getMethod("setForeground", mSetForegroundSignature);
		} catch (NoSuchMethodException e) {
			throw new IllegalStateException("OS doesn't have Service.startForeground OR Service.setForeground!");
		}
	}

	@Override
	public IBinder onBind(Intent intent) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {

		if (!RUNTIME_STATUS_ACTIVE) {
			startScheduler();
			RUNTIME_STATUS_ACTIVE = true;
		}
		return super.onStartCommand(intent, flags, startId);
	}

	@Override
	public void onDestroy() {
		// Make sure our notification is gone.
		stopForegroundCompat(ID);
		if (RUNTIME_STATUS_ACTIVE) {
			RUNTIME_STATUS_ACTIVE = false;
			stopScheduler();
		}
		super.onDestroy();
	}

	private void startScheduler() {
		// if (logger == null) {
		// logger = new AndroidLogger3(this);
		// Logger.setLogger(logger);
		// }

		/*
		 * On a machine, only a single transporter can run on the same port. If
		 * another port is used than the default (proposed), the port must be
		 * specified when configuring the addresses.
		 */
		if (port == PORT_NOT_SET) {
			port = 52000;
		}

		runtime = createRuntime();

		// SOAPSerializer serializer = new SOAPSerializer();
		// String endpoint = SocketTransporter.SCHEME + "://" + (address!=null ?
		// address.getHostAddress() : "localhost");
		// transporter = new SocketTransporter(port, port, endpoint, runtime,
		// serializer);
		// runtime.registerMessagedEndpoint(SocketTransporter.SCHEME,
		// transporter);
		// transporter.start();

		runtime.start();

		runtime.setContext("Android", this);
		Handler arctisUpdateHandler = new Handler();
		runtime.setContext("Handler", arctisUpdateHandler);
		runtime.setContext("ServiceController", (ForegroundController) this);

		// logger.runtimeSystemStarted();

		runtime.setTerminationListener(new Runtime.IRuntimeTerminationListener() {
			public void runtimeStopped() {
				// logger.runtimeSystemStopped();
				port = PORT_NOT_SET;
				// TODO: stop service
				// stopScheduler();
				stopForegroundCompat(ID);
			}
		});
	}

	private void stopScheduler() {
		// logger.runtimeSystemStopped();
		runtime.stop();
		port = PORT_NOT_SET;
	}

	protected abstract Runtime createRuntime();

	protected InetAddress getLocalIpAddress() {
		try {
			for (Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces(); en.hasMoreElements();) {
				NetworkInterface intf = en.nextElement();
				for (Enumeration<InetAddress> enumIpAddr = intf.getInetAddresses(); enumIpAddr.hasMoreElements();) {
					InetAddress inetAddress = enumIpAddr.nextElement();
					if (!inetAddress.isLoopbackAddress()) {
						return inetAddress; // .getHostAddress().toString();
					}
				}
			}
		} catch (SocketException ex) {
			ex.printStackTrace();
		}
		return null;
	}

}