package no.ntnu.item.arctis.examples.android.geirsag.rtp.rtcpreceiver;

import java.io.IOException;
import java.io.OutputStream;
import java.net.DatagramSocket;
import java.net.InetSocketAddress;
import java.net.SocketException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Timer;
import java.util.TimerTask;

import jlibrtp.DataFrame;
import jlibrtp.Participant;
import jlibrtp.RTCPAppIntf;
import jlibrtp.RTPAppIntf;
import jlibrtp.RTPSession;
import no.ntnu.item.arctis.examples.android.geirsag.rtp.constants.RTPConstants;
import no.ntnu.item.arctis.examples.android.geirsag.rtp.objects.RTPReceiverBundle;
import no.ntnu.item.arctis.runtime.android.AndroidBlock;
import android.util.Log;

public class RTCPReceiver extends AndroidBlock {
	private RTPSession rtpSession = null;
	private DatagramSocket rtpSocket = null;
	private DatagramSocket rtcpSocket = null;
	private OutputStream outputStream = null;
	private int byteCounter;
	private int printCounter = 0;
	private final int MILESTONE = 10000; // 10kB
	private final static int DEBUG = 1;
	private final static String TAG = "RTCP Receiver";

	private HashMap<InetSocketAddress, Long> receiverTimeout;
	private Timer timer;
	private HashMap<InetSocketAddress, TimerTask> timerTasks;

	public Exception initialize(RTPReceiverBundle bundle) {
		this.outputStream = bundle.outputStream;
		int port = bundle.getPort();
		try {
			rtpSocket = new DatagramSocket(port);
			rtcpSocket = new DatagramSocket(port + 1);
			Log.d(TAG, "Sockets " + port + " and " + (port + 1) + "open");
		} catch (SocketException e) {
			Log.e(TAG, "Could not open sockets: " + port + ", " + (port + 1));
			return e;
		}
		
		receiverTimeout = new HashMap<InetSocketAddress, Long>();
		timerTasks = new HashMap<InetSocketAddress, TimerTask>();
		timer = new Timer(true);
		rtpSession = new RTPSession(rtpSocket, rtcpSocket);
		rtpSession.naivePktReception(true); // accept packets from anyone
		rtpSession.RTPSessionRegister(rtpHandler, rtcpHandler, null);
		return null;
	}

	public void stopListening() {
		timer.cancel();
		rtpSession.endSession();
		rtpSocket.close();
		rtcpSocket.close();
	}

	private RTCPAppIntf rtcpHandler = new RTCPAppIntf() {

		@Override
		public void SRPktReceived(long ssrc, long ntpHighOrder, long ntpLowOrder, long rtpTimestamp, long packetCount,
				long octetCount, long[] reporteeSsrc, int[] lossFraction, int[] cumulPacketsLost, long[] extHighSeq,
				long[] interArrivalJitter, long[] lastSRTimeStamp, long[] delayLastSR) {
			// TODO Auto-generated method stub

		}

		@Override
		public void SDESPktReceived(Participant[] relevantParticipants) {
			// TODO Auto-generated method stub

		}

		@Override
		public void RRPktReceived(long reporterSsrc, long[] reporteeSsrc, int[] lossFraction, int[] cumulPacketsLost,
				long[] extHighSeq, long[] interArrivalJitter, long[] lastSRTimeStamp, long[] delayLastSR) {
			// TODO Auto-generated method stub

		}

		@Override
		public void BYEPktReceived(Participant[] relevantParticipants, String reason) {
			if (DEBUG > 0)
				Log.d(TAG, String.format("Received BYE packet from: %s, Reason: %s",
						Arrays.toString(relevantParticipants), reason));
		}

		@Override
		public void APPPktReceived(Participant part, int subtype, byte[] name, byte[] data) {
			if (DEBUG > 0)
				Log.d(TAG, String.format("Received APP packet from: %s, subtype: %i, name: %s, data: %s",
						part.getNAME(), subtype, new String(name), new String(data)));
			if (name.equals(RTPConstants.END_OF_STREAM_BYTES)) {
				InetSocketAddress rtpAddress = part.getRtpSocketAddress();
				if (rtpAddress != null) {
					timerTasks.get(rtpAddress).cancel();
					timerTasks.remove(rtpAddress);
					receiverTimeout.remove(rtpAddress);
					sendToBlock("MSG_RECEIVE_END", rtpAddress);
				} else {
					Log.e(TAG, "Received End Of Stream packet from participant with no RTP address!");
				}
			}
		}
	};

	private RTPAppIntf rtpHandler = new RTPAppIntf() {

		public void receiveData(DataFrame frame, Participant participant) {
			try {
				if (DEBUG > 0) {
					byteCounter += frame.getConcatenatedData().length;
					if (byteCounter / MILESTONE > printCounter) {
						printCounter++;
						Log.d(TAG, "Received " + byteCounter + " bytes");
					}
				}

				InetSocketAddress rtpAddress = participant.getRtpReceivedFromAddress();
				if (rtpAddress != null) {
					if (!receiverTimeout.containsKey(rtpAddress)) {
						sendToBlock("MSG_RECEIVE_START", rtpAddress);
						TimerTask task = new MyTask(rtpAddress);
						timerTasks.put(rtpAddress, task);
						timer.schedule(task, 1000, 1000);
					}
					receiverTimeout.put(rtpAddress, System.currentTimeMillis());
				} else {
					Log.e(TAG, "RtpReceivedFromAddress is null!");
				}

				outputStream.write(frame.getConcatenatedData());
			} catch (IOException e) {
				Log.e(TAG,
						"Error writing received RTP frame to output stream, message: " + e.getLocalizedMessage(), e);
			}
		}

		public void userEvent(int type, Participant[] participant) {
			// TODO Auto-generated method stub

		}

		public int frameSize(int payloadType) {
			// TODO Auto-generated method stub
			return 1;
		}

	};

	private class MyTask extends TimerTask {

		private InetSocketAddress rtpAddress;
		static final long maxInterval = 2 * 1000;

		MyTask(InetSocketAddress rtpAddress) {
			this.rtpAddress = rtpAddress;
		}

		@Override
		public void run() {
			// TODO Auto-generated method stub
			if (System.currentTimeMillis() - receiverTimeout.get(rtpAddress) > maxInterval) {
				timerTasks.remove(rtpAddress);
				receiverTimeout.remove(rtpAddress);
				sendToBlock("MSG_RECEIVE_TIMEOUT", rtpAddress);
				cancel();
			}
		}

	}

}
