package no.ntnu.item.arctis.examples.android.geirsag.rtp.rtpreceiver2;

import java.io.IOException;
import java.io.OutputStream;
import java.net.DatagramSocket;
import java.net.SocketException;

import jlibrtp.DataFrame;
import jlibrtp.Participant;
import jlibrtp.RTPAppIntf;
import jlibrtp.RTPSession;
import no.ntnu.item.arctis.examples.android.geirsag.rtp.objects.RTPReceiverBundle;
import no.ntnu.item.arctis.runtime.android.AndroidBlock;
import android.util.Log;

public class RTPReceiver2 extends AndroidBlock {
	private RTPSession rtpSession = null;
	private DatagramSocket rtpSocket = null;
	private DatagramSocket rtcpSocket = null;
	private OutputStream outputStream = null;
	private int byteCounter;
	private int printCounter = 0;
	private final int MILESTONE = 10000; // 10kB
	private final static int DEBUG = 1;

	public boolean initialize(RTPReceiverBundle bundle) {
		this.outputStream = bundle.outputStream;
		int port = bundle.getPort();
		try {
			rtpSocket = new DatagramSocket(port);
			rtcpSocket = new DatagramSocket(port + 1);
			Log.d("RTP", "Sockets " + port + " and " + (port + 1) + "open");
		} catch (SocketException e) {
			Log.e("RTP", "Could not open sockets: " + port + ", " + (port + 1));
			return false;
		}

		rtpSession = new RTPSession(rtpSocket, rtcpSocket);
		rtpSession.naivePktReception(true); // accept packets from anyone
		rtpSession.RTPSessionRegister(new RTPHandler(), null, null);
		return true;
	}

	public void stopListening() {
		rtpSession.endSession();
		rtpSocket.close();
		rtcpSocket.close();
	}

	private class RTPHandler implements RTPAppIntf {

		public void receiveData(DataFrame frame, Participant participant) {
			try {
				if (DEBUG > 0) {
					byteCounter += frame.getConcatenatedData().length;
					if (byteCounter / MILESTONE > printCounter) {
						printCounter++;
						Log.d("RTPReceiver", "Received " + byteCounter + " bytes");
					}
				}
				outputStream.write(frame.getConcatenatedData());
			} catch (IOException e) {
				Log.e("IOException",
						"Error writing received RTP frame to output stream, message: " + e.getLocalizedMessage(), e);
			}
		}

		public void userEvent(int type, Participant[] participant) {
			// TODO Auto-generated method stub

		}

		public int frameSize(int payloadType) {
			// TODO Auto-generated method stub
			return 1;
		}

	}
}