package no.ntnu.item.arctis.examples.android.geirsag.rtp.rtpreceiver3;

import java.io.IOException;
import java.io.OutputStream;
import java.net.DatagramSocket;
import java.net.InetSocketAddress;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Timer;
import java.util.TimerTask;

import jlibrtp.DataFrame;
import jlibrtp.Participant;
import jlibrtp.RTPAppIntf;
import jlibrtp.RTPSession;
import no.ntnu.item.arctis.examples.android.geirsag.rtp.constants.RTPConstants;
import no.ntnu.item.arctis.examples.android.geirsag.rtp.objects.MessageBundle;
import no.ntnu.item.arctis.runtime.android.AndroidBlock;
import android.util.Log;

import com.Ostermiller.util.CircularByteBuffer;

public class RTPReceiver3 extends AndroidBlock {
	private RTPSession rtpSession = null;
	private DatagramSocket rtpSocket = null;
	private DatagramSocket rtcpSocket = null;
	private int byteCounter;
	private int printCounter = 0;
	private final int MILESTONE = 10000; // 10kB
	private final static long MAX_INTERVAL = 500;
	private final static long TIMER_DELAY = 1000;
	private final static long TIMER_PERIOD = 1000;
	private final static int DEBUG = 1;
	private final static String TAG = "RTP Receiver";

	private Timer timer;
	private HashMap<InetSocketAddress, Sender> senders;

	public Exception initialize(int port) {
		try {
			rtpSocket = new DatagramSocket(port);
			rtcpSocket = new DatagramSocket(port + 1);
			Log.d(TAG, "Sockets " + port + " and " + (port + 1) + "open");
		} catch (SocketException e) {
			Log.e(TAG, "Could not open sockets: " + port + ", " + (port + 1));
			return e;
		}

		senders = new HashMap<InetSocketAddress, Sender>();
		timer = new Timer(true);
		rtpSession = new RTPSession(rtpSocket, rtcpSocket);
		rtpSession.naivePktReception(true); // accept packets from anyone
		rtpSession.RTPSessionRegister(rtpHandler, null, null);
		return null;
	}

	public void stopListening() {
		timer.cancel();
		rtpSession.endSession();
		rtpSocket.close();
		rtcpSocket.close();
		closeStreams();
	}

	private void closeStreams() {
		ArrayList<Sender> senderList = new ArrayList<Sender>(senders.values());
		for (Sender sender : senderList) {
			sender.close();
			senders.remove(sender);
		}
	}

	private void countBytes(DataFrame frame) {
		byteCounter += frame.getConcatenatedData().length;
		if (byteCounter / MILESTONE > printCounter) {
			printCounter++;
			Log.d(TAG, "Received " + byteCounter + " bytes");
		}
	}


	private RTPAppIntf rtpHandler = new RTPAppIntf() {

		public void receiveData(DataFrame frame, Participant participant) {
			try {
				if (DEBUG > 0) {
					countBytes(frame);
				}

				InetSocketAddress rtpAddress = participant.getRtpReceivedFromAddress();
				if (rtpAddress != null) {
					if (!senders.containsKey(rtpAddress)) {
						senders.put(rtpAddress, new Sender(rtpAddress));
					}
					senders.get(rtpAddress).lastTimestamp = System.currentTimeMillis();
					byte[] data = frame.getConcatenatedData();
					if (data.length == RTPConstants.END_OF_STREAM_BYTES.length && Arrays.equals(data, RTPConstants.END_OF_STREAM_BYTES)) {
						if (DEBUG > 0)
							Log.d(TAG, "Received end of stream code in RTP stream");
						senders.get(rtpAddress).close();
						senders.remove(rtpAddress);
					} else
						senders.get(rtpAddress).out.write(data);
				} else {
					Log.e(TAG, "RtpReceivedFromAddress is null!");
				}

			} catch (IOException e) {
				Log.e(TAG, "Error writing received RTP frame to output stream, message: " + e.getLocalizedMessage(), e);
			}
		}

		public void userEvent(int type, Participant[] participant) {
		}

		public int frameSize(int payloadType) {
			return 1;
		}

	};

	private class Sender {
		public long lastTimestamp;
		public final OutputStream out;
		public final InetSocketAddress rtpAddress;
		TimerTask timeoutTask = new TimerTask() {

			@Override
			public void run() {
				if (System.currentTimeMillis() - lastTimestamp > MAX_INTERVAL) {
					close();
					senders.remove(rtpAddress);
					if (DEBUG > 0)
						Log.d(TAG, String.format("Has not received packet from %s in %f seconds, timing out",
								rtpAddress, MAX_INTERVAL / 1000.0));
				}
			}
		};
		public Sender(final InetSocketAddress rtpAddress) {
			this.rtpAddress = rtpAddress;
			
			timer.schedule(timeoutTask, TIMER_DELAY, TIMER_PERIOD);
			CircularByteBuffer cbb = new CircularByteBuffer(-1, false);
			sendToBlock("MSG_RECEIVE_START", new MessageBundle(rtpAddress, cbb.getInputStream()));
			this.out = cbb.getOutputStream();
		}

		public void close() {
			timeoutTask.cancel();
			Log.d(TAG, "Closing stream");
			try {
				out.write(-1); // End of stream
				out.flush();
				out.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

	}
}