package no.ntnu.item.arctis.examples.android.geirsag.rtp.rtpsender2;

import java.io.IOException;
import java.io.InputStream;
import java.net.DatagramSocket;
import java.net.SocketException;

import jlibrtp.DataFrame;
import jlibrtp.Participant;
import jlibrtp.RTPAppIntf;
import jlibrtp.RTPSession;
import no.ntnu.item.arctis.examples.android.geirsag.rtp.objects.RTPSenderBundle;
import no.ntnu.item.arctis.runtime.android.AndroidBlock;
import android.util.Log;

public class RTPSender2 extends AndroidBlock {

	private RTPSession rtpSession = null;
	private DatagramSocket rtpSocket = null;
	private DatagramSocket rtcpSocket = null;
	private volatile boolean isSending = false;
	private InputStream inputStream = null;
	private int byteCounter;
	private int printCounter = 0;
	private final int MILESTONE = 10000; // 10kB
	private final static int DEBUG = 1;

	private Thread senderThread;
	
	private class SenderThread extends Thread {
		public void run() {
			Log.d("RTPSender", "Sender thread started");
			isSending = true;
			byte[] buffer = new byte[bufferSize];
			int bytesRead;
			while (isSending) {
				try {
					bytesRead = inputStream.read(buffer);

					if (DEBUG > 0) {
						byteCounter += bytesRead;
						if (byteCounter / MILESTONE > printCounter) {
							printCounter++;
							Log.d("RTPSender", byteCounter + " bytes sent");
						}
					}

					if (bytesRead < 0)
						isSending = false;
					else if (bytesRead < bufferSize) {
						byte[] croppedBuffer = new byte[bytesRead];
						for (int i = 0; i < croppedBuffer.length; i++) {
							croppedBuffer[i] = buffer[i];
						}
						rtpSession.sendData(croppedBuffer);
					} else {
						rtpSession.sendData(buffer);
					}
				} catch (IOException e) {
					e.printStackTrace();
					isSending = false;
				}
			}
			try {
				inputStream.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			Log.d("RTPSender", "Sender thread stopping");
		}
	};
	// Instance parameter. Edit only in overview page.
	public final int bufferSize;

	public boolean initialize(RTPSenderBundle input) {
		if(senderThread != null && senderThread.isAlive()){
			Log.e("RTPSender", "Trying to initialize RTP Sender while sender thread is still alive!");
			return false;
		}

		try {
			rtpSocket = new DatagramSocket(input.getReceiverPort());
			rtcpSocket = new DatagramSocket(input.getReceiverPort() + 1);
			Log.d("RTP", "Sockets open at ports: " + input.getReceiverPort() + " and " + (input.getReceiverPort() + 1));
		} catch (SocketException e) {
			Log.e("RTP", "Could not open sockets: " + input.getReceiverPort() + ", " + (input.getReceiverPort() + 1));
			return false;
		}

		inputStream = input.getInputStream();

		rtpSession = new RTPSession(rtpSocket, rtcpSocket);
		rtpSession.RTPSessionRegister(new RTPHandler(), null, null);
		Participant p = new Participant(input.getDestinationAddress(), input.getDestinationPort(),
				input.getDestinationPort() + 1);
		rtpSession.addParticipant(p);

		senderThread = new SenderThread();
		senderThread.start();
		return true;
	}

	public void stopSession() {
		isSending = false;
		try {
			senderThread.join();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		rtpSession.endSession();
		rtpSocket.close();
		rtcpSocket.close();
	}

	private class RTPHandler implements RTPAppIntf {

		public void receiveData(DataFrame frame, Participant participant) {
			// should not be receiving data
			Log.w("RTP", "Received unexpected data from: " + participant.getCNAME());
		}

		public void userEvent(int type, Participant[] participant) {

		}

		public int frameSize(int payloadType) {
			return 1;
		}
	}

	// Do not edit this constructor.
	public RTPSender2(int bufferSize) {
		this.bufferSize = bufferSize;
	}

}