package no.ntnu.item.arctis.examples.android.geirsag.rtp.rtpsender3;

import java.io.IOException;
import java.io.InputStream;
import java.net.DatagramSocket;
import java.net.SocketException;
import java.util.Arrays;

import jlibrtp.DataFrame;
import jlibrtp.Participant;
import jlibrtp.RTPAppIntf;
import jlibrtp.RTPSession;
import no.ntnu.item.arctis.examples.android.geirsag.rtp.constants.RTPConstants;
import no.ntnu.item.arctis.examples.android.geirsag.rtp.objects.RTPSenderBundle;
import no.ntnu.item.arctis.runtime.android.AndroidBlock;
import android.util.Log;

public class RTPSender3 extends AndroidBlock {

	private RTPSession rtpSession;
	private DatagramSocket rtpSocket;
	private DatagramSocket rtcpSocket;
	private InputStream inputStream;
	private volatile boolean isSending = false;
	

	private static final String TAG = "RTP Sender 3";
	
	// Instance parameter. Edit only in overview page.
	public final int bufferSize;

	private Thread senderThread;
	
	private class SenderThread extends Thread {

		public void run() {
			isSending = true;
			byte[] buffer = new byte[bufferSize];
			int bytesRead;
			while (isSending) {
				try {
					bytesRead = inputStream.read(buffer);
					if (bytesRead < 0){
						isSending = false;
						rtpSession.sendData(RTPConstants.END_OF_STREAM_BYTES);
					}
					else if(bytesRead < bufferSize) {
						byte[] croppedBuffer = new byte[bytesRead];
						for (int i = 0; i < croppedBuffer.length; i++) {
							croppedBuffer[i] = buffer[i];
						}
						rtpSession.sendData(croppedBuffer);
					} else {
						rtpSession.sendData(buffer);
					}
				} catch (IOException e) {
					e.printStackTrace();
					isSending = false;
				}
			}
			try {
				inputStream.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			rtpSession.endSession();
		}
	};
	
	public Exception initialize(RTPSenderBundle input) {
		try {
			rtpSocket = new DatagramSocket(input.getReceiverPort());
			rtcpSocket = new DatagramSocket(input.getReceiverPort() + 1);
			Log.d(TAG, "Sockets open at ports: " + input.getReceiverPort() + " and " + (input.getReceiverPort() + 1));
		} catch (SocketException e) {
			Log.e(TAG, "Could not open sockets: " + input.getReceiverPort() + ", " + (input.getReceiverPort() + 1));
			return e;
		}
		
		inputStream = input.getInputStream();

		rtpSession = new RTPSession(rtpSocket, rtcpSocket);
		rtpSession.RTPSessionRegister(rtpHandler, null, null);
		Participant p = new Participant(input.getDestinationAddress(), input.getDestinationPort(),
				input.getDestinationPort() + 1);
		rtpSession.addParticipant(p);
		
		senderThread = new SenderThread();
		senderThread.start();
		return null;
	}
	
	public void stopSession() {
		isSending = false;
		try {
			senderThread.join();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		rtpSession.endSession();
		rtpSocket.close();
		rtcpSocket.close();
	}
	
	private RTPAppIntf rtpHandler = new RTPAppIntf() {
		
		@Override
		public void userEvent(int type, Participant[] participant) {
			Log.d(TAG, String.format("Received userEvent %i from participants %s", type, Arrays.toString(participant)));
		}
		
		@Override
		public void receiveData(DataFrame frame, Participant participant) {
			// should not be receiving data
			Log.w(TAG, "Received unexpected data from: " + participant.getCNAME());
		}
		
		@Override
		public int frameSize(int payloadType) {
			return 1;
		}
	};

	// Do not edit this constructor.
	public RTPSender3(int bufferSize) {
	    this.bufferSize = bufferSize;
	}

}
