package no.ntnu.item.arctis.examples.android.geirsag.xmpp.xmppclient4;

import java.io.Serializable;
import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.StringTokenizer;

import no.ntnu.item.arctis.examples.android.geirsag.xmpp.objects.ChatMessage;
import no.ntnu.item.arctis.examples.android.geirsag.xmpp.objects.CustomMessage;
import no.ntnu.item.arctis.examples.android.geirsag.xmpp.objects.LoginBundle;
import no.ntnu.item.arctis.examples.android.geirsag.xmpp.objects.StatusBundle;
import no.ntnu.item.arctis.runtime.Block;

import org.jivesoftware.smack.Chat;
import org.jivesoftware.smack.ChatManager;
import org.jivesoftware.smack.ChatManagerListener;
import org.jivesoftware.smack.ConnectionConfiguration;
import org.jivesoftware.smack.ConnectionListener;
import org.jivesoftware.smack.MessageListener;
import org.jivesoftware.smack.PacketListener;
import org.jivesoftware.smack.Roster;
import org.jivesoftware.smack.RosterEntry;
import org.jivesoftware.smack.RosterListener;
import org.jivesoftware.smack.XMPPConnection;
import org.jivesoftware.smack.XMPPException;
import org.jivesoftware.smack.filter.PacketFilter;
import org.jivesoftware.smack.packet.Message;
import org.jivesoftware.smack.packet.Packet;
import org.jivesoftware.smack.packet.Presence;
import org.jivesoftware.smack.packet.Presence.Mode;
import org.jivesoftware.smack.packet.Presence.Type;
import org.jivesoftware.smackx.commands.AdHocCommandManager;

public class XMPPClient4 extends Block {

	private XMPPConnection connection;
	private ChatManager chatManager;
	private Roster roster;

	private MessageListener myMessageListener = new MessageListener() {

		public void processMessage(Chat chat, Message message) {
			sendToBlock("MESSAGE_RECEIVED", new ChatMessage(message.getTo(), chat.getThreadID(), message.getBody()));
		}
	};
	// Instance parameter. Edit only in overview page.
	public final java.lang.String customMessages;

	public void connectToServer(final LoginBundle bundle) {
		ConnectionConfiguration config = new ConnectionConfiguration(bundle.host, bundle.port, bundle.serviceName);
		connection = new XMPPConnection(config);
		// AdHocCommandManager adHocManager =
		// AdHocCommandManager.getAddHocCommandsManager(connection);
		Runnable r = new Runnable() {
			public void run() {
				try {
					connection.connect();
					log("XMPP: Connected to XMPP server: " + bundle.host);
					String username = bundle.username.contains("@") ? bundle.username : bundle.username + "@gmail.com";
					connection.login(username, bundle.password);
					log("XMPP: Successfully logged in with username: " + bundle.username);
					connection.addConnectionListener(new ConnectionListener() {

						public void connectionClosed() {
							sendToBlock("CONNECTION_CLOSED", null);
						}

						public void connectionClosedOnError(Exception e) {
							sendToBlock("CONNECTION_CLOSED", e);
						}

						public void reconnectingIn(int seconds) {
							log("XMPP: XMPP Connection reconnecting in " + seconds + " seconds");
						}

						public void reconnectionFailed(Exception e) {
							log("XMPP: XMPP Connection reconnection failed", e);
						}

						public void reconnectionSuccessful() {
							log("XMPP: XMPP Connection reconnected successfully");
						}
					});
					sendToBlock("LOGIN_SUCCESSFUL");
				} catch (XMPPException e) {
					log("XMPP: Could not connect or login to server: " + bundle.host + ", message: "
							+ e.getLocalizedMessage());
					sendToBlock("LOGIN_FAILED", e);
				}
			}
		};
		Thread t = new Thread(r);
		t.start();
	}

	public void setupListeners() {
		// Presence setup
		this.roster = connection.getRoster();
		roster.addRosterListener(new RosterListener() {

			public void entriesAdded(Collection<String> addresses) {
				log("XMPPClient: Roster entries added: " + Arrays.toString(addresses.toArray()));
				sendToBlock("ROSTER_CHANGED");
			}

			public void entriesDeleted(Collection<String> addresses) {
				log("XMPPClient: Roster entries deleted: " + Arrays.toString(addresses.toArray()));
				sendToBlock("ROSTER_CHANGED");
			}

			public void entriesUpdated(Collection<String> addresses) {
				log("XMPPClient: Roster entries updated: " + Arrays.toString(addresses.toArray()));
				sendToBlock("ROSTER_CHANGED");
			}

			public void presenceChanged(Presence presence) {
				String user = presence.getFrom();
				Presence bestPresence = roster.getPresence(user);
				sendToBlock("PRESENCE_CHANGED", bestPresence);
			}

		});

		// Chat setup
		this.chatManager = connection.getChatManager();
		chatManager.addChatListener(new ChatManagerListener() {

			public void chatCreated(Chat chat, boolean createdLocally) {
				log("XMPP: Chat created with user: " + chat.getParticipant() + ". Created locally: " + createdLocally);
				if (!createdLocally)
					chat.addMessageListener(myMessageListener);
			}
		});

		StringTokenizer st = new StringTokenizer(customMessages, ",");
		while (st.hasMoreTokens()) {
			registerMsgListener(st.nextToken());
		}
	}

	public XMPPConnection getConnection() {
		return connection;
	}

	public void disconnect() {
		connection.disconnect();
		//return null;
	}

	// Presence
	public Presence getPresence(String userID) {
		return roster.getPresence(userID);
	}

	public Collection<Presence> getAllPresences() {
		Collection<Presence> presences = new ArrayList<Presence>();
		Collection<RosterEntry> entries = roster.getEntries();
		for (RosterEntry e : entries) {
			presences.add(roster.getPresence(e.getUser()));
		}
		return presences;
	}

	public void setPresence(StatusBundle bundle) {
		connection.sendPacket(new Presence(Presence.Type.available, bundle.status, 0, bundle.presenceMode));
	}

	public Collection<RosterEntry> getContacts() {
		Collection<RosterEntry> contacts = roster.getEntries();
		return contacts;
	}

	// Chat
	public void sendMessage(ChatMessage chatMessage) {
		Chat chat = null;
		String userJID = chatMessage.getToUser();
		String message = chatMessage.getMessage();
		try {
			chat = chatManager.getThreadChat(userJID);
		} catch (Exception e) {
			// No chat exists yet
		}
		if (chat == null) {
			chat = chatManager.createChat(userJID, userJID, myMessageListener);
		}
		try {
			chat.sendMessage(message);
		} catch (XMPPException e) {
			log("XMPP: Error sending message to user: " + userJID, e);
			// return e;
		}
		// return null;
	}

	public void registerMsgListener(final String messageName) {
		connection.addPacketListener(new PacketListener() {

			public void processPacket(Packet packet) {
				CustomMessage msg = new CustomMessage(messageName, (Serializable) packet.getProperty(messageName), packet.getTo(),
						packet.getFrom());
				log(String.format("XMPPClient: Received custom message: %s", msg.toString()));
				sendToBlock("CUSTOM_MSG_RECEIVED", msg);
			}
		}, new PacketFilter() {

			public boolean accept(Packet packet) {
				if (packet.getPropertyNames().contains(messageName))
					return true;
				else
					return false;
			}
		});
	}

	public void sendCustomMsg(CustomMessage customMsg) {
		// Packet msg = new Presence(Presence.Type.available, null, 0,
		// Mode.available);
		// msg.
		Packet packet = new Message(customMsg.getToUser(), Message.Type.chat);
		// Packet presence = new Presence(Type.available);
		// presence.setTo(customMsg.getToUser());
		packet.setFrom(connection.getUser());
		packet.setProperty(customMsg.getName(), customMsg.getValue());
		try {
			connection.sendPacket(packet);
			log(String.format("XMPPClient: Custom message sent: %s", customMsg));
		} catch (Exception e) {
			log("XMPP: Error sending custom message: " + customMsg, e);
//			return e;
		}
//		return null;
	}

	public void sendCustomMultiple(Collection<CustomMessage> messages) {
		try {
			for (CustomMessage msg : messages) {
				Packet p = new Message(msg.getToUser(), Message.Type.chat);
				p.setFrom(connection.getUser());
				p.setProperty(msg.getName(), msg.getValue());
				connection.sendPacket(p);

				log(String.format("XMPPClient: Custom message sent: %s", msg));
			}
		} catch (Exception e) {
			log("XMPP: Error sending multiple custom messages", e);
//			return e;
		}
//		return null;
	}

	// Do not edit this constructor.
	public XMPPClient4(java.lang.String customMessages) {
		this.customMessages = customMessages;
	}

}
