package no.ntnu.item.arctis.examples.android.geirsag.xmpp.xmppclient5;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.StringTokenizer;

import no.ntnu.item.arctis.examples.android.geirsag.xmpp.objects.ChatMessage;
import no.ntnu.item.arctis.examples.android.geirsag.xmpp.objects.CustomMessage;
import no.ntnu.item.arctis.examples.android.geirsag.xmpp.objects.LoginBundle;
import no.ntnu.item.arctis.examples.android.geirsag.xmpp.objects.StatusBundle;
import no.ntnu.item.arctis.runtime.Block;

import org.jivesoftware.smack.Chat;
import org.jivesoftware.smack.ChatManager;
import org.jivesoftware.smack.ChatManagerListener;
import org.jivesoftware.smack.ConnectionConfiguration;
import org.jivesoftware.smack.ConnectionListener;
import org.jivesoftware.smack.MessageListener;
import org.jivesoftware.smack.PacketListener;
import org.jivesoftware.smack.ReconnectionManager;
import org.jivesoftware.smack.Roster;
import org.jivesoftware.smack.RosterEntry;
import org.jivesoftware.smack.RosterListener;
import org.jivesoftware.smack.XMPPConnection;
import org.jivesoftware.smack.XMPPException;
import org.jivesoftware.smack.filter.PacketFilter;
import org.jivesoftware.smack.packet.Message;
import org.jivesoftware.smack.packet.Packet;
import org.jivesoftware.smack.packet.Presence;
import org.jivesoftware.smack.util.StringUtils;

public class XMPPClient5 extends Block {

	private XMPPConnection connection;
	private ChatManager chatManager;
	private Roster roster;

	private volatile static boolean rosterReceived = false;

	private final Runnable getRoster = new Runnable() {
		@Override
		public void run() {
			sendToBlock("ROSTER_CHANGED", roster.getEntries());
		}
	};

	private MessageListener myMessageListener = new MessageListener() {

		public void processMessage(Chat chat, Message message) {
			if (message.getBody() != null && message.getBody().length() > 0)
				sendToBlock(
						"MESSAGE_RECEIVED",
						new ChatMessage(message.getTo(), StringUtils.parseBareAddress(message.getFrom()), message
								.getBody()));
		}
	};
	// Instance parameter. Edit only in overview page.
	public final java.lang.String customMessages;

	public void connectToServer(final LoginBundle bundle) {
		ConnectionConfiguration config = new ConnectionConfiguration(bundle.host, bundle.port, bundle.serviceName);
		connection = new XMPPConnection(config);
		Runnable r = new Runnable() {
			public void run() {
				try {
					connection.connect();
					log("XMPP: Connected to XMPP server: " + bundle.host);
					String username = bundle.username.contains("@") ? bundle.username : bundle.username + "@gmail.com";
					connection.login(username, bundle.password);
					log("XMPP: Successfully logged in with username: " + bundle.username);
					connection.addConnectionListener(new ConnectionListener() {

						public void connectionClosed() {
							sendToBlock("CONNECTION_CLOSED", null);
						}

						public void connectionClosedOnError(Exception e) {
							sendToBlock("CONNECTION_CLOSED", e);
						}

						public void reconnectingIn(int seconds) {
							log("XMPP: XMPP Connection reconnecting in " + seconds + " seconds");
						}

						public void reconnectionFailed(Exception e) {
							log("XMPP: XMPP Connection reconnection failed", e);
						}

						public void reconnectionSuccessful() {
							log("XMPP: XMPP Connection reconnected successfully");
						}
					});
					setupListeners();
					sendToBlock("LOGIN_SUCCESSFUL");
				} catch (XMPPException e) {
					log("XMPP: Could not connect or login to server: " + bundle.host + ", message: "
							+ e.getLocalizedMessage());
					sendToBlock("LOGIN_FAILED", e);
				}
			}
		};
		Thread t = new Thread(r);
		t.start();
	}

	public void setupListeners() {
		// Presence setup
		this.roster = connection.getRoster();

		// If a roster update is not received within 1 sec, request one.
		rosterReceived = false;
		new Thread() {
			@Override
			public void run() {
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
				}
				if (!rosterReceived) {
					rosterReceived = true;
					new Thread(getRoster).start();
				}
			}
		}.start();

		roster.addRosterListener(new RosterListener() {

			public void entriesAdded(Collection<String> addresses) {
				log("XMPPClient: Roster entries added: " + Arrays.toString(addresses.toArray()));
				rosterReceived = true;
				new Thread(getRoster).start();
			}

			public void entriesDeleted(Collection<String> addresses) {
				log("XMPPClient: Roster entries deleted: " + Arrays.toString(addresses.toArray()));
				new Thread(getRoster).start();
			}

			public void entriesUpdated(Collection<String> addresses) {
				log("XMPPClient: Roster entries updated: " + Arrays.toString(addresses.toArray()));
				new Thread(getRoster).start();
			}

			public void presenceChanged(Presence presence) {
				final String user = presence.getFrom();
				new Thread(new Runnable() {
					public void run() {
						Presence bestPresence = roster.getPresence(user);
						sendToBlock("PRESENCE_CHANGED", bestPresence);
					}
				}).start();
			}

		});

		// Chat setup
		this.chatManager = connection.getChatManager();
		chatManager.addChatListener(new ChatManagerListener() {

			public void chatCreated(Chat chat, boolean createdLocally) {
				log("XMPP: Chat created with user: " + chat.getParticipant() + ". Created locally: " + createdLocally);
				if (!createdLocally)
					chat.addMessageListener(myMessageListener);
			}
		});

		StringTokenizer st = new StringTokenizer(customMessages, ",");
		while (st.hasMoreTokens()) {
			registerMsgListener(st.nextToken().trim());
		}
	}

	public Exception disconnect() {
		connection.disconnect();
		return null;
	}

	public void setPresence(final StatusBundle bundle) {
		new Thread(new Runnable() {
			public void run() {
				if(connection.isConnected())
					connection.sendPacket(new Presence(Presence.Type.available, bundle.status, 0, bundle.presenceMode));
				else
					log("XMPP: Error setting presence: Not connected!");
			}
		}).start();
	}

	// Chat
	public void sendMessage(final ChatMessage chatMessage) {
		new Thread(new Runnable() {
			public void run() {
				Chat chat = null;
				String userJID = chatMessage.getToUser();
				String message = chatMessage.getMessage();
				try {
					chat = chatManager.getThreadChat(userJID);
				} catch (Exception e) {
					// No chat exists yet
				}
				if (chat == null) {
					chat = chatManager.createChat(userJID, userJID, myMessageListener);
				}
				try {
					chat.sendMessage(message);
				} catch (XMPPException e) {
					log("XMPP: Error sending message to user: " + userJID, e);
				}
			}
		}).start();
	}

	public void registerMsgListener(final String messageName) {
		connection.addPacketListener(new PacketListener() {

			public void processPacket(Packet packet) {
				CustomMessage msg = new CustomMessage(messageName, (Serializable) packet.getProperty(messageName),
						packet.getTo(), packet.getFrom());
				log(String.format("XMPPClient: Received custom message: %s", msg.toString()));
				sendToBlock("CUSTOM_MSG_RECEIVED", msg);
			}
		}, new PacketFilter() {

			public boolean accept(Packet packet) {
				if (packet.getPropertyNames().contains(messageName))
					return true;
				else
					return false;
			}
		});
	}

	public void sendCustomMsg(final CustomMessage customMsg) {
		final Packet packet = new Message(customMsg.getToUser(), Message.Type.chat);
		packet.setFrom(connection.getUser());
		try {
			packet.setProperty(customMsg.getName(), customMsg.getValue());
		} catch (Exception e) {
			log("XMPP: Error sending custom message: " + customMsg, e);
			return;
		}
		new Thread(new Runnable() {
			public void run() {
				try {
					connection.sendPacket(packet);
					log(String.format("XMPPClient: Custom message sent: %s", customMsg));
				} catch (Exception e) {
					log("XMPP: Error sending custom message: " + customMsg, e);
				}
			}
		}).start();
	}

	public void sendCustomMultiple(final Collection<CustomMessage> messages) {
		new Thread(new Runnable() {
			public void run() {
				try {
					for (CustomMessage msg : messages) {
						Packet p = new Message(msg.getToUser(), Message.Type.chat);
						p.setFrom(connection.getUser());
						p.setProperty(msg.getName(), msg.getValue());
						connection.sendPacket(p);

						log(String.format("XMPPClient: Custom message sent: %s", msg));
					}
				} catch (Exception e) {
					log("XMPP: Error sending multiple custom messages", e);
				}
			}
		}).start();
	}

	// Do not edit this constructor.
	public XMPPClient5(java.lang.String customMessages) {
		this.customMessages = customMessages;
	}

	public void requestUpdates() {
		new Thread(new Runnable() {
			public void run() {
				Collection<RosterEntry> entries = roster.getEntries();
				sendToBlock("ROSTER_CHANGED", entries);
				for (RosterEntry rosterEntry : entries) {
					sendToBlock("PRESENCE_CHANGED", roster.getPresence(rosterEntry.getUser()));
				}
			}
		}).start();
	}

}