package no.ntnu.item.arctis.examples.android.geirsag.xmpp.xmppexchangeipaddress;

import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Collection;
import java.util.Enumeration;

import no.ntnu.item.arctis.examples.android.geirsag.xmpp.objects.UserIPBundle;
import no.ntnu.item.arctis.examples.android.geirsag.xmpp.xmppsetup.MyConnectionListener;
import no.ntnu.item.arctis.runtime.Block;

import org.jivesoftware.smack.PacketListener;
import org.jivesoftware.smack.XMPPConnection;
import org.jivesoftware.smack.filter.PacketFilter;
import org.jivesoftware.smack.packet.Message;
import org.jivesoftware.smack.packet.Packet;

public class XMPPExchangeIPAddress extends Block {

	public static final String IP_ADDRESS_REQUEST = "IP_ADDRESS_REQUEST";
	public static final String IP_ADDRESS_RESPONSE = "IP_ADDRESS_RESPONSE";

	private XMPPConnection connection;

	public void initialize(XMPPConnection connection) {
		this.connection = connection;
		connection.addPacketListener(new PacketListener() {
			public void processPacket(Packet packet) {
				InetAddress ip = null;
				Collection<String> properties = packet.getPropertyNames();
				try {
					if (properties.contains(IP_ADDRESS_REQUEST)) {
						ip = (InetAddress) packet.getProperty(IP_ADDRESS_REQUEST);
						sendToBlock("IP_REQUEST", new UserIPBundle(ip, packet.getFrom()));
					} else if (properties.contains(IP_ADDRESS_RESPONSE)) {
						ip = (InetAddress) packet.getProperty(IP_ADDRESS_RESPONSE);
						sendToBlock("IP_RESPONSE", new UserIPBundle(ip, packet.getFrom()));
					}
				} catch (Exception e) {
					log("XMPP: Error parsing IP property", e);
				}
			}
		}, new PacketFilter() {
			public boolean accept(Packet packet) {
				return !(packet.getProperty(IP_ADDRESS_REQUEST) == null || packet.getProperty(IP_ADDRESS_RESPONSE) == null);
			}
		});
		connection.addConnectionListener(new MyConnectionListener(blockID));
	}

	public void requestIp(String userJID) {
		Message p = new Message(userJID);
		p.setBody("- IP request from " + connection.getUser() + " -");
		p.setProperty(IP_ADDRESS_REQUEST, getLocalIpAddress());
		connection.sendPacket(p);
	}

	public void respondWithIp(UserIPBundle bundle) {
		Message p = new Message(bundle.getUserJID());
		p.setBody("- IP response from " + connection.getUser() + " -");
		p.setProperty(IP_ADDRESS_RESPONSE, getLocalIpAddress());
		connection.sendPacket(p);
	}

	public InetAddress getLocalIpAddress() {
		try {
			for (Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces(); en.hasMoreElements();) {
				NetworkInterface intf = en.nextElement();
				for (Enumeration<InetAddress> enumIpAddr = intf.getInetAddresses(); enumIpAddr.hasMoreElements();) {
					InetAddress inetAddress = enumIpAddr.nextElement();
					if (!inetAddress.isLoopbackAddress()) {
						return inetAddress;
					}
				}
			}
		} catch (SocketException ex) {
			log("XMPP: Could not get local IP address!", ex);
		}
		return null;
	}
}
